# groceryAggregator-be

## Introduction

This project is a backend part of an application designed to compare selected online grocery e-shops operating in the Czech Republic. Its main goal is to facilitate the comparison of food prices and the subsequent shopping process across different e-shops. Utilizing Spring Boot for the backend, PostgreSQL for data storage, and Keycloak for authentication, the system connects to external e-shops via REST API to aggregate and present data. This setup provides a robust backend infrastructure capable of supporting complex data processing and integration needs for the grocery aggregator application.

## Technologies

This project is built using the following technologies:

- **Java 17**: The core programming language used.
- **Spring Boot 2.7.7**: Framework for creating standalone, production-grade Spring-based applications with minimal effort.
- **Spring Web MVC & WebFlux**: For creating RESTful web services.
- **Lombok**: For reducing boilerplate code in Java.
- **Spring Data JPA**: For database access and management.
- **PostgreSQL**: As the relational database for storing data.
- **Spring Security & OAuth2 Resource Server**: For securing the application and implementing authentication and authorization.
- **Apache HttpClient**: For making HTTP requests to external web services.
- **Spring Boot Mail**: For sending emails.

## Requirements

The project requires Java 17 or newer, PostgreSQL, and Keycloak for authentication. Ensure these are installed and properly configured before running the application.

## Installation

1. Clone the repository to your local machine.
2. Ensure PostgreSQL is running and create a database for the application.
3. Configure Keycloak for authentication. Detailed instructions are provided in the Keycloak Configuration section below.
4. Based on the environment you intend to run, ensure the database connection settings and other necessary configurations are correctly set in the respective YAML file.

### Configuration Details

#### Keycloak Configuration

The application uses Keycloak for authentication and authorization. It is crucial to correctly configure the Keycloak server addresses (issuer-uri & jwk-set-uri) in the specific YAML configuration files for different environments:

- `application-dev-live.yaml` for the live development environment.
- `application-dev-mock.yaml` for the mock development environment.
- `application-production.yaml` for the production environment (now is commented).

Don't forget to start the docker Keycloak resource server using the included scripts (see README in the appropriate folder).

#### Application Configuration Highlights

- **Common Configuration (`application.yaml`)**: 
    - uses PostgreSQL as the database and sets the default database schema to `grocery_schema` 
    - configures Spring Mail with Mailtrap for email testing
        - email sending can be tested using curl and pop3 connection (e.g. this command prints all messages: `curl --ssl-reqd -u username:password pop3://pop3.mailtrap.io:9950` - credentials are in the application.yaml file)
        - email configurations such as `EMAIL_FROM` and `EMAIL_SUBJECT` can be adjusted in the `Constants.java` file
    - sets the server port to 8081

- **Development Environments (`application-dev-live.yaml` & `application-dev-mock.yaml`)**: 
    - enables server error messages
    - sets logging level to:
        - `DEBUG` for `cz.gottlieb.groceryAggregator` 
        - `TRACE` for `org.springframework.security`
    - configures Spring Security OAuth2 Resource Server with JWT 
    - sets the URL of the Spring datasource to a local PostgreSQL database (with port 5432 and database name: `groceryAggregator`)
        - for development purposes the `ddl-auto` attribute is set to `create-drop` 

- **Production Environment (`application-production.yaml`)**: 
    - This file NEEDS to be set up according to YOUR environment! DO NOT use it without changes in the file!
        - includes TODO comments for necessary changes for the production environment
        - maintains the same Spring Mail configuration as the development environment
            - make sure you have the correct settings and that you are not still using the mocking service `mailtrap` unless you really want it (!)
            - ensure to review `Constants.java`
        - configures the URL of Spring datasource for the Azure PostgreSQL database with SSL (this is an example!)
            - make sure that the `ddl-auto` attribute is set correctly according to your expectations (!)
                - for production environment the `validate` setting is certainly much more appropriate

## Build

To build the project using Maven, navigate to the root directory of the project and run the following command:

`mvnw clean compile`

This command will compile the project and prepare it for running.

## Running the Application

To run the application without using an IDE, you can use Maven commands directly from your terminal. Navigate to the root directory of your project and execute the Maven command for the desired profile:

- Development environment with live settings: `mvnw spring-boot:run -Dspring-boot.run.profiles=dev-live`
- Development environment with mock settings: `mvnw spring-boot:run -Dspring-boot.run.profiles=dev-mock`
- Production environment: `mvnw spring-boot:run -Dspring-boot.run.profiles=production`

Ensure your application properties or YAML files match the settings required for the active profile.

## Usage

Once the application is running, you can interact with the backend directly through its REST API endpoints. Detailed documentation of these endpoints is provided within the project.

## License

This project is created solely for educational purposes and is not intended for distribution or commercial use. All content is provided without any warranties.

## Authors and Acknowledgments

This project was developed by Jan Gottlieb as part of a diploma thesis.