package cz.gottlieb.groceryAggregator.configs.restTemplate;

import cz.gottlieb.groceryAggregator.utils.MyErrorHandler;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.config.CookieSpecs;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.ssl.SSLContextBuilder;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;
import org.springframework.http.client.ClientHttpRequestFactory;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.context.annotation.SessionScope;

import javax.net.ssl.SSLContext;
import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.util.List;

@Slf4j
@Configuration
public class RohlikRestTemplateConfig {

    private final RohlikInterceptor rohlikInterceptor;

    @Value("${trust.store.file}")
    private Resource trustStoreFile;

    @Value("${trust.store.password}")
    private String trustStorePassword;

    @Value("${connection.timeout}")
    private int timeout;

    public RohlikRestTemplateConfig(RohlikInterceptor rohlikInterceptor) {
        this.rohlikInterceptor = rohlikInterceptor;
    }

    @Bean
    @SessionScope
    public RestTemplate rohlikRestTemplate() throws IOException, CertificateException, NoSuchAlgorithmException, KeyStoreException, KeyManagementException {

        RequestConfig config = RequestConfig.custom()
                /**
                 * The RFC 6265 compliant policy (interoprability profile).
                 */
                .setCookieSpec(CookieSpecs.STANDARD)
                /**
                 * How long to wait for a connection to be established with the remote server before throwing a timeout exception.
                 */
                .setConnectTimeout(timeout)
                /**
                 *   How long to wait when trying to checkout a connection from the connection pool before throwing an exception
                 *   (the connection pool won't return immediately if, for example, all the connections are checked out).
                 */
                .setConnectionRequestTimeout(timeout)
                /**
                 * How long to wait for the server to respond to various calls before throwing a timeout exception.
                 * See http://docs.oracle.com/javase/1.5.0/docs/api/java/net/SocketOptions.html#SO_TIMEOUT for details.
                 */
                .setSocketTimeout(timeout)
                .build();

        SSLContext sslContext = new SSLContextBuilder()
                .loadTrustMaterial(trustStoreFile.getURL(), trustStorePassword.toCharArray())
                .build();

        SSLConnectionSocketFactory sslConFactory = new SSLConnectionSocketFactory(sslContext);

        CloseableHttpClient httpClient = HttpClientBuilder.create()
                .setDefaultRequestConfig(config)
                .setSSLSocketFactory(sslConFactory)
                .build();

        ClientHttpRequestFactory requestFactory = new HttpComponentsClientHttpRequestFactory(httpClient);

        RestTemplate restTemplate = new RestTemplate(requestFactory);

        List<ClientHttpRequestInterceptor> interceptors = restTemplate.getInterceptors();
        interceptors.add(rohlikInterceptor);

//        restTemplate.setMessageConverters(createMessageConverters());
        restTemplate.setErrorHandler(new MyErrorHandler());

        return restTemplate;
    }

}
