package cz.gottlieb.groceryAggregator.controllers;

import cz.gottlieb.groceryAggregator.configs.connectors.BillaConfig;
import cz.gottlieb.groceryAggregator.configs.connectors.RohlikConfig;
import cz.gottlieb.groceryAggregator.persistence.model.Company;
import cz.gottlieb.groceryAggregator.persistence.model.GroceryCredential;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingList;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.persistence.repository.GroceryCredentialRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.ShoppingListRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.UserRepository;
import cz.gottlieb.groceryAggregator.services.CompanyService;
import cz.gottlieb.groceryAggregator.utils.GroceryEnum;
import cz.gottlieb.groceryAggregator.utils.ShoppingListEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.Environment;
import org.springframework.core.env.Profiles;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.PostConstruct;

/**
 * Editing User and Company entities is possible ONLY with ADMIN rights.
 */
@Slf4j
@Transactional
@RestController
@PreAuthorize("hasRole('admin')")
@RequestMapping("/admin")
public class AdminController {
    private final GroceryCredentialRepository groceryCredentialRepository;
    private final CompanyService companyService;
    private final UserRepository userRepository;
    private final ShoppingListRepository shoppingListRepository;
    private final RohlikConfig rohlikConfig;
    private final BillaConfig billaConfig;
    private final Environment environment;

    private Company rohlikDbEntity;
    private Company billaDbEntity;

    public AdminController(GroceryCredentialRepository groceryCredentialRepository,
                           CompanyService companyService,
                           UserRepository userRepository,
                           ShoppingListRepository shoppingListRepository,
                           RohlikConfig rohlikConfig,
                           BillaConfig billaConfig,
                           Environment environment) {
        this.groceryCredentialRepository = groceryCredentialRepository;
        this.companyService = companyService;
        this.userRepository = userRepository;
        this.shoppingListRepository = shoppingListRepository;
        this.rohlikConfig = rohlikConfig;
        this.billaConfig = billaConfig;
        this.environment = environment;
    }

    @PostConstruct
    private void initCompany() {
        log.info(">>> initCompany");

        rohlikDbEntity = companyService.setUpCompany(GroceryEnum.ROHLIK, rohlikConfig);
        billaDbEntity = companyService.setUpCompany(GroceryEnum.BILLA, billaConfig);

        if (!environment.acceptsProfiles(Profiles.of("production"))) {
            makeTestData();
        }
    }


    private void makeTestData() {
        log.info(">>> makeTestData");

        User userFranta = new User();
        userFranta.setSsoUuid("b21197e3-67d5-40f8-8174-a49fcac5070a");
        userFranta.setEmail("franta@example.com");
        userFranta.setUsername("myuser");
        userFranta.setFirstName("Franta");
        userFranta.setLastName("Vomáčka");
        userFranta.getCredentials();
        userFranta.getShoppingLists();
        userFranta.getWatchDogs();    // TODO fill it
        userFranta.getProductGroups();    // TODO not yet implemented
        userRepository.save(userFranta);

        // rohlik
        GroceryCredential credFrantaRohlik = new GroceryCredential();
        credFrantaRohlik.setCompany(rohlikDbEntity);
        credFrantaRohlik.setUsername("petr.sklenicka01@seznam.cz");
        credFrantaRohlik.setPassword("rohlicek123456+");
        groceryCredentialRepository.save(credFrantaRohlik);

        // billa
        GroceryCredential credFrantaBilla = new GroceryCredential();
        credFrantaBilla.setCompany(billaDbEntity);
        credFrantaBilla.setUsername("petr.sklenicka01@seznam.cz");
        credFrantaBilla.setPassword("billacz123456+");
        groceryCredentialRepository.save(credFrantaBilla);

        // kosik etc.
        // ...

        userFranta.getCredentials().add(credFrantaRohlik);
        userFranta.getCredentials().add(credFrantaBilla);
        userRepository.save(userFranta);

        ShoppingList defaultCart = new ShoppingList();
        defaultCart.setName(ShoppingListEnum.DEFAULT_CART.name());
        defaultCart.setType(ShoppingListEnum.DEFAULT_CART);
        shoppingListRepository.save(defaultCart);

        ShoppingList birthdayWishList = new ShoppingList();
        birthdayWishList.setName("Birthday wishes");
        birthdayWishList.setType(ShoppingListEnum.WISHLIST);
        shoppingListRepository.save(birthdayWishList);

        userFranta.getShoppingLists().add(defaultCart);
        userFranta.getShoppingLists().add(birthdayWishList);
        userRepository.save(userFranta);
    }

}
