package cz.gottlieb.groceryAggregator.controllers;

import cz.gottlieb.groceryAggregator.annotation.CurrentUser;
import cz.gottlieb.groceryAggregator.dto.fe.FeGroceryCredentialDto;
import cz.gottlieb.groceryAggregator.dto.mapper.DtoFeMapper;
import cz.gottlieb.groceryAggregator.persistence.model.Company;
import cz.gottlieb.groceryAggregator.persistence.model.GroceryCredential;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.services.CompanyService;
import cz.gottlieb.groceryAggregator.services.CredentialsService;
import cz.gottlieb.groceryAggregator.services.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Comparator;
import java.util.List;

@Slf4j
@Transactional
@RestController
@PreAuthorize("hasRole('user') and hasAuthority('SCOPE_read')")
@RequestMapping("/grocery-credential")
public class CredentialController {

    private final UserService userService;
    private final CredentialsService credentialsService;
    private final CompanyService companyService;

    public CredentialController(UserService userService,
                                CredentialsService credentialsService,
                                CompanyService companyService) {
        this.userService = userService;
        this.credentialsService = credentialsService;
        this.companyService = companyService;
    }

    @GetMapping
    public ResponseEntity<List<FeGroceryCredentialDto>> getAllGroceryCredentials(@CurrentUser User user) {
        log.info(">>> getAllGroceryCredentials");

        List<GroceryCredential> credentials = user.getCredentials().stream()
                .sorted(Comparator.comparing(GroceryCredential::getId)).toList();

        return new ResponseEntity<>(DtoFeMapper.mapToFeGroceryCredentialDto(credentials), HttpStatus.OK);
    }

    @GetMapping("/{id}")
    public ResponseEntity<FeGroceryCredentialDto> getGroceryCredentialById(@PathVariable Long id, @CurrentUser User user) {
        log.info(">>> getGroceryCredentialById");

        GroceryCredential credential = credentialsService.findCredentialsByIdOrThrow(user, id);

        return new ResponseEntity<>(DtoFeMapper.mapToFeGroceryCredentialDto(credential), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PostMapping
    public ResponseEntity<FeGroceryCredentialDto> createGroceryCredential(@Valid @RequestBody FeGroceryCredentialDto feCredentialRequest,
                                                                          @CurrentUser User user) {
        log.info(">>> createGroceryCredential");

        Company company = companyService.findCompanyByNameOrThrow(feCredentialRequest.getCompany());

        GroceryCredential credentialReq = DtoFeMapper.mapToGroceryCredential(feCredentialRequest, company);
        credentialReq.setId(null);

//        credentialReq.setCompany(company);
        credentialsService.save(credentialReq);

        user.getCredentials().add(credentialReq);
//        userService.save(user); // TODO not mandatory?
        return new ResponseEntity<>(DtoFeMapper.mapToFeGroceryCredentialDto(credentialReq), HttpStatus.CREATED);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PutMapping("/{id}")
    public ResponseEntity<FeGroceryCredentialDto> updateGroceryCredential(@PathVariable Long id,
                                                                          @Valid @RequestBody FeGroceryCredentialDto updatedFeCredentialReq,
                                                                          @CurrentUser User user) {
        log.info(">>> updateGroceryCredential");

        GroceryCredential credential = credentialsService.findCredentialsByIdOrThrow(user, id);

        Company company = companyService.findCompanyByNameOrThrow(updatedFeCredentialReq.getCompany());

        GroceryCredential updatedCredentialReq = DtoFeMapper.mapToGroceryCredential(updatedFeCredentialReq, company);
        updatedCredentialReq.setId(credential.getId());
//        updatedCredentialReq.setCompany(company);

        credentialsService.save(updatedCredentialReq);
        return new ResponseEntity<>(DtoFeMapper.mapToFeGroceryCredentialDto(updatedCredentialReq), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteGroceryCredential(@PathVariable Long id, @CurrentUser User user) {
        log.info(">>> deleteGroceryCredential");

        GroceryCredential credential = credentialsService.findCredentialsByIdOrThrow(user, id);

        user.getCredentials().remove(credential);
        userService.save(user);

        credentialsService.deleteById(id);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

}
