package cz.gottlieb.groceryAggregator.controllers;

import com.fasterxml.jackson.core.JsonProcessingException;
import cz.gottlieb.groceryAggregator.annotation.CurrentUser;
import cz.gottlieb.groceryAggregator.dto.fe.FeProductResponseDto;
import cz.gottlieb.groceryAggregator.dto.mapper.DtoFeMapper;
import cz.gottlieb.groceryAggregator.persistence.model.GroceryCredential;
import cz.gottlieb.groceryAggregator.persistence.model.Product;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.persistence.repository.ProductRepository;
import cz.gottlieb.groceryAggregator.services.connectors.GroceryService;
import cz.gottlieb.groceryAggregator.services.impl.CredentialsServiceImpl;
import cz.gottlieb.groceryAggregator.utils.GroceryEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.HttpStatusCodeException;

import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

@Slf4j
@Transactional
@RestController
@PreAuthorize("hasRole('user') and hasAuthority('SCOPE_read')")
@RequestMapping("/grocery")
public class GroceryController {
    public static final int MAX_ITEM_PER_GROCERY = 10; // Max 2*10 items for 2 groceryEshop
    public static final String SERVICE_NAME_SUFFIX = "ServiceImpl";
    //
    private final Map<String, GroceryService> groceryServiceMap;
    private final CredentialsServiceImpl credentialsService;
    private final ProductRepository productRepository;

    public GroceryController(CredentialsServiceImpl credentialsService,
                             Map<String, GroceryService> groceryServiceMap,
                             ProductRepository productRepository) {
        this.credentialsService = credentialsService;
        this.groceryServiceMap = groceryServiceMap;
        this.productRepository = productRepository;
    }

    @GetMapping("/findByName")
    public ResponseEntity<List<FeProductResponseDto>> findByName(@RequestParam String name, @RequestParam Optional<String> filteredCompany,
                                                                 @CurrentUser User user) {
        log.info(">>> findByName");

        List<Product> aggregatedProducts = new ArrayList<>();

        AtomicInteger productMaxSize = new AtomicInteger(0);

        List<Iterator<Product>> productsIterators = new LinkedList<>();

        try {
            for (GroceryEnum groceryEnum : GroceryEnum.values()) {
                if (filteredCompany.isPresent() && Objects.equals(GroceryEnum.valueOf(filteredCompany.get()), groceryEnum)) {
                    continue; // find products from other Companies
                }

                GroceryService groceryService = groceryServiceMap.get(groceryEnum.name().toLowerCase() + SERVICE_NAME_SUFFIX); // eg. rohlikServiceImpl

                Optional<GroceryCredential> credential = credentialsService.findCredentialsByGroceryName(user, groceryEnum);

                if (credential.isEmpty()) {
                    continue;
                }

                List<Product> products = groceryService.findProductsByName(name, credential.get());
                productsIterators.add(products.iterator());
                productMaxSize.updateAndGet(currentInt -> Math.max(currentInt, products.size()));
            }

            int size = Math.min(MAX_ITEM_PER_GROCERY, productMaxSize.get());

            for (int i = 0; i < size; i++) {
                for (Iterator<Product> it : productsIterators) {
                    if (it.hasNext()) {
                        aggregatedProducts.add(it.next());
                    }
                }
            }
        } catch (HttpStatusCodeException exception) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeProductResponseDto(aggregatedProducts), exception.getStatusCode());
        } catch (JsonProcessingException e) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeProductResponseDto(aggregatedProducts), HttpStatus.INTERNAL_SERVER_ERROR);
        } catch (RuntimeException e) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeProductResponseDto(aggregatedProducts), HttpStatus.INTERNAL_SERVER_ERROR);
        }

        return new ResponseEntity<>(DtoFeMapper.mapToFeProductResponseDto(aggregatedProducts), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PostMapping("/findByNameAndSku/save")
    public ResponseEntity<FeProductResponseDto> findByNameAndSku(@RequestParam String name,
                                                                 @RequestParam String originSku,
                                                                 @RequestParam String company,
                                                                 @CurrentUser User user) {
        log.info(">>> findByNameAndSku");

        Product newProduct;
        GroceryEnum groceryEnum = GroceryEnum.valueOf(company);

        try {
            GroceryService groceryService = groceryServiceMap.get(groceryEnum.name().toLowerCase() + SERVICE_NAME_SUFFIX); // eg. rohlikServiceImpl

            GroceryCredential credential = credentialsService.findCredentialsByGroceryNameOrThrow(user, groceryEnum);

            // check lastUpdate
            Optional<Product> optProd = productRepository.findByOriginSkuAndCompany_Name(originSku, groceryEnum);
            if (optProd.isPresent() && optProd.get().getUpdateAt().isAfter(LocalDateTime.now().minusHours(24))) {
                return new ResponseEntity<>(DtoFeMapper.mapToFeProductResponseDto(optProd.get()), HttpStatus.OK);
            }

            newProduct = groceryService.findProductByNameAndSkuOrThrow(name, originSku, credential);

            productRepository.findByOriginSkuAndCompany_Name(originSku, groceryEnum)
                    .ifPresent(productEntityDb -> newProduct.setId(productEntityDb.getId()));   // Product update

            productRepository.save(newProduct);

        } catch (HttpStatusCodeException exception) {
            return new ResponseEntity<>(null, exception.getStatusCode());
        }

        return new ResponseEntity<>(DtoFeMapper.mapToFeProductResponseDto(newProduct), HttpStatus.OK);
    }

}
