package cz.gottlieb.groceryAggregator.controllers;

import com.fasterxml.jackson.core.JsonProcessingException;
import cz.gottlieb.groceryAggregator.annotation.CurrentUser;
import cz.gottlieb.groceryAggregator.dto.fe.FeShoppingListRequestDto;
import cz.gottlieb.groceryAggregator.dto.fe.FeShoppingListResponseDto;
import cz.gottlieb.groceryAggregator.dto.mapper.DtoFeMapper;
import cz.gottlieb.groceryAggregator.persistence.model.GroceryCredential;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingList;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingListItem;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.persistence.repository.ShoppingListItemRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.ShoppingListRepository;
import cz.gottlieb.groceryAggregator.services.CredentialsService;
import cz.gottlieb.groceryAggregator.services.ShoppingListService;
import cz.gottlieb.groceryAggregator.services.UserService;
import cz.gottlieb.groceryAggregator.services.connectors.GroceryService;
import cz.gottlieb.groceryAggregator.utils.GroceryEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.util.Pair;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.HttpStatusCodeException;
import org.springframework.web.server.ResponseStatusException;

import javax.validation.Valid;
import java.util.*;
import java.util.concurrent.ThreadLocalRandom;

@Slf4j
@Transactional
@RestController
@PreAuthorize("hasRole('user') and hasAuthority('SCOPE_read')")
@RequestMapping("/shopping-lists")
public class ShoppingListController {
    public static final String SERVICE_NAME_SUFFIX = "ServiceImpl";
    //
    private final ShoppingListItemRepository shoppingListItemRepository;
    private final Map<String, GroceryService> groceryServiceMap;
    private final UserService userService;
    private final ShoppingListRepository shoppingListRepository;
    private final ShoppingListService shoppingListService;
    private final CredentialsService credentialsService;

    public ShoppingListController(UserService userService,
                                  ShoppingListRepository shoppingListRepository,
                                  ShoppingListService shoppingListService,
                                  CredentialsService credentialsService,
                                  ShoppingListItemRepository shoppingListItemRepository,
                                  Map<String, GroceryService> groceryServiceMap) {
        this.userService = userService;
        this.shoppingListRepository = shoppingListRepository;
        this.shoppingListService = shoppingListService;
        this.credentialsService = credentialsService;
        this.shoppingListItemRepository = shoppingListItemRepository;
        this.groceryServiceMap = groceryServiceMap;
    }

    @GetMapping
    public ResponseEntity<List<FeShoppingListResponseDto>> getAllShoppingLists(@CurrentUser User user) {
        log.info(">>> getAllShoppingLists");

        List<ShoppingList> selectedShoppingList = shoppingListService.findAllWishLists(user).stream()
                .sorted(Comparator.comparing(ShoppingList::getId)).toList();

        return new ResponseEntity<>(DtoFeMapper.mapToFeShoppingListDto(selectedShoppingList), HttpStatus.OK);
    }

    @GetMapping("/{id}")
    public ResponseEntity<FeShoppingListResponseDto> getShoppingListById(@PathVariable Long id, @CurrentUser User user) {
        log.info(">>> getShoppingListById");

        ShoppingList selectedShoppingList = shoppingListService.findWishListByIdOrThrow(user, id);

        return new ResponseEntity<>(DtoFeMapper.mapToFeShoppingListDto(selectedShoppingList), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PostMapping
    public ResponseEntity<FeShoppingListResponseDto> createShoppingList(@Valid @RequestBody FeShoppingListRequestDto feShoppingListRequestDto,
                                                                        @CurrentUser User user) {
        log.info(">>> createShoppingList");

        findShoppingListDuplicityByName(user.getShoppingLists(), feShoppingListRequestDto.getName())
                .ifPresent(duplicatedShoppingListName -> {
                    throw new ResponseStatusException(HttpStatus.BAD_REQUEST,
                            "ShoppingList (name: " + duplicatedShoppingListName.getName() + ") already exists!");    // TODO change exception
                });

        ShoppingList createdShoppingList = DtoFeMapper.mapToShoppingList(feShoppingListRequestDto);

        shoppingListRepository.save(createdShoppingList);

        user.getShoppingLists().add(createdShoppingList);
        userService.save(user);
        return new ResponseEntity<>(DtoFeMapper.mapToFeShoppingListDto(createdShoppingList), HttpStatus.CREATED);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PutMapping("/{id}")
    public ResponseEntity<FeShoppingListResponseDto> updateShoppingList(@PathVariable Long id,
                                                                        @Valid @RequestBody FeShoppingListRequestDto feShoppingListRequestDto,
                                                                        @CurrentUser User user) {
        log.info(">>> updateShoppingList");

        shoppingListService.findWishListByIdOrThrow(user, id);

        findShoppingListDuplicityByName(user.getShoppingLists(), feShoppingListRequestDto.getName())
                .ifPresent(duplicatedShoppingListName -> {
                    throw new ResponseStatusException(HttpStatus.BAD_REQUEST,
                            "ShoppingList (name: " + duplicatedShoppingListName.getName() + ") already exists!");    // TODO change exception
                });

        ShoppingList updatedShoppingList = DtoFeMapper.mapToShoppingList(feShoppingListRequestDto);
        updatedShoppingList.setId(id);

        shoppingListRepository.save(updatedShoppingList);
        return new ResponseEntity<>(DtoFeMapper.mapToFeShoppingListDto(updatedShoppingList), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteShoppingList(@PathVariable Long id, @CurrentUser User user) {
        log.info(">>> deleteShoppingList");

        ShoppingList selectedShoppingList = shoppingListService.findWishListByIdOrThrow(user, id);

        user.getShoppingLists().remove(selectedShoppingList);
        userService.save(user);

        shoppingListRepository.deleteById(id);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PostMapping("/{id}/copy-from-carts")
    public ResponseEntity<FeShoppingListResponseDto> copyFromCarts(@PathVariable Long id, @CurrentUser User user) {
        log.info(">>> copyFromCarts");

        ShoppingList selectedShoppingList = shoppingListService.findWishListByIdOrThrow(user, id);

        try {
            List<ShoppingListItem> removedItems = new ArrayList<>();

            for (GroceryEnum groceryEnum : GroceryEnum.values()) {
                GroceryService groceryService = groceryServiceMap.get(groceryEnum.name().toLowerCase() + SERVICE_NAME_SUFFIX); // eg. rohlikServiceImpl

                Optional<GroceryCredential> credential = credentialsService.findCredentialsByGroceryName(user, groceryEnum);
                if (credential.isEmpty()) {
                    continue;
                }

                HttpStatus loginStatus = groceryService.login(credential.get());
                if (!loginStatus.is2xxSuccessful()) {
                    continue;
                }

                Pair<HttpStatus, List<ShoppingListItem>> result = groceryService.loadCart();
                List<ShoppingListItem> items = result.getSecond();

                shoppingListService.saveOrUpdate(selectedShoppingList, items);

                selectedShoppingList.getShoppingListItems().stream()
                        .filter(item -> Objects.equals(item.getProduct().getCompany().getName(), groceryEnum))
                        .filter(item -> !items.contains(item))
                        .forEach(item -> {
                            shoppingListItemRepository.delete(item);
                            removedItems.add(item);
                        });
            }

            selectedShoppingList.getShoppingListItems().removeAll(removedItems);
            shoppingListRepository.save(selectedShoppingList);
        } catch (HttpStatusCodeException exception) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeShoppingListDto(selectedShoppingList), exception.getStatusCode());
        } catch (JsonProcessingException e) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeShoppingListDto(selectedShoppingList), HttpStatus.INTERNAL_SERVER_ERROR);
        }

        return new ResponseEntity<>(DtoFeMapper.mapToFeShoppingListDto(selectedShoppingList), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PostMapping("/{id}/copy-to-carts")
    public ResponseEntity<Map<String, Integer>> copyToCarts(@PathVariable Long id, @CurrentUser User user) {
        log.info(">>> copyToCarts");

        ShoppingList selectedShoppingList = shoppingListService.findWishListByIdOrThrow(user, id);

        Map<String, Integer> resultErrors = new HashMap<>();
        try {
            for (GroceryEnum groceryEnum : GroceryEnum.values()) {
                GroceryService groceryService = groceryServiceMap.get(groceryEnum.name().toLowerCase() + SERVICE_NAME_SUFFIX); // eg. rohlikServiceImpl

                Optional<GroceryCredential> credential = credentialsService.findCredentialsByGroceryName(user, groceryEnum);
                if (credential.isEmpty()) {
                    continue;
                }

                int groceryErrors = clearAndFillCart(groceryService, credential.get(), selectedShoppingList);

                resultErrors.put(groceryEnum.name() + "_ERRORS:", groceryErrors);
            }
        } catch (JsonProcessingException e) {
            return new ResponseEntity<>(resultErrors, HttpStatus.INTERNAL_SERVER_ERROR);
        }
        int sumErrors = resultErrors.values().stream()
                .reduce(0, Integer::sum);

        HttpStatus retStatus = (sumErrors == 0 ? HttpStatus.OK : HttpStatus.INTERNAL_SERVER_ERROR);
        return new ResponseEntity<>(resultErrors, retStatus);
    }

    private int clearAndFillCart(GroceryService groceryService, GroceryCredential credential, ShoppingList selectedShoppingList) throws JsonProcessingException {
        log.info(">>> getAllShoppingLists");
        int errors = 0;

        HttpStatus loginStatus = groceryService.login(credential);

        if (!groceryService.loadCart().getSecond().isEmpty()) {
            groceryService.clearCart();
        }

        if (loginStatus.is2xxSuccessful()) {
            for (ShoppingListItem item : selectedShoppingList.getShoppingListItems()) {
                if (item.getProduct().getCompany().getName() != groceryService.getType()) {
                    continue;
                }
                try {
                    if (!groceryService.addToCart(item).getFirst().is2xxSuccessful()) {
                        errors++;
                    }
                    Thread.sleep(ThreadLocalRandom.current().nextInt(800, 1600));
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
            }
        }
        return errors;
    }

    private Optional<ShoppingList> findShoppingListDuplicityByName(List<ShoppingList> shoppingLists, String name) {
        return shoppingLists.stream()
                .filter(sl -> Objects.equals(sl.getName(), name))
                .findFirst();
    }

}
