package cz.gottlieb.groceryAggregator.controllers;

import cz.gottlieb.groceryAggregator.annotation.CurrentUser;
import cz.gottlieb.groceryAggregator.dto.fe.FeItemRequestDto;
import cz.gottlieb.groceryAggregator.dto.fe.FeItemResponseDto;
import cz.gottlieb.groceryAggregator.dto.mapper.DtoFeMapper;
import cz.gottlieb.groceryAggregator.persistence.model.Product;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingList;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingListItem;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.persistence.repository.ProductRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.ShoppingListItemRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.ShoppingListRepository;
import cz.gottlieb.groceryAggregator.services.ProductService;
import cz.gottlieb.groceryAggregator.services.ShoppingListService;
import cz.gottlieb.groceryAggregator.services.UserService;
import cz.gottlieb.groceryAggregator.services.connectors.GroceryService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import javax.validation.Valid;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Slf4j
@Transactional
@RestController
@PreAuthorize("hasRole('user') and hasAuthority('SCOPE_read')")
@RequestMapping("/shopping-lists")
public class ShoppingListItemController {

    private final UserService userService;
    private final ShoppingListRepository shoppingListRepository;
    private final ShoppingListItemRepository shoppingListItemRepository;
    private final ShoppingListService shoppingListService;
    private final ProductRepository productRepository;
    private final Map<String, GroceryService> groceryServiceMap;
    private final ProductService productService;

    public ShoppingListItemController(UserService userService,
                                      ShoppingListRepository shoppingListRepository,
                                      ShoppingListItemRepository shoppingListItemRepository,
                                      ShoppingListService shoppingListService,
                                      ProductRepository productRepository,
                                      ProductService productService,
                                      Map<String, GroceryService> groceryServiceMap) {
        this.userService = userService;
        this.shoppingListRepository = shoppingListRepository;
        this.shoppingListItemRepository = shoppingListItemRepository;
        this.shoppingListService = shoppingListService;
        this.productRepository = productRepository;
        this.groceryServiceMap = groceryServiceMap;
        this.productService = productService;
    }

    @GetMapping("/{listId}/items")
    public ResponseEntity<List<FeItemResponseDto>> getAllShoppingListItems(@PathVariable Long listId, @CurrentUser User user) {

        ShoppingList shoppingList = shoppingListService.findWishListByIdOrThrow(user, listId);
        List<ShoppingListItem> shoppingListItems = shoppingList.getShoppingListItems().stream()
                .sorted(Comparator.comparing(ShoppingListItem::getId)).toList();

        return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(shoppingListItems), HttpStatus.OK);
    }

    @GetMapping("/{listId}/items/{idItems}")
    public ResponseEntity<FeItemResponseDto> getShoppingListItemById(@PathVariable Long listId, @PathVariable Long idItems,
                                                                     @CurrentUser User user) {

        ShoppingList shoppingList = shoppingListService.findWishListByIdOrThrow(user, listId);
        ShoppingListItem shoppingListItem = findShoppingListItemByIdOrThrow(shoppingList, idItems);

        return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(shoppingListItem), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PostMapping("/{listId}/items")
    public ResponseEntity<FeItemResponseDto> createShoppingListItem(@PathVariable Long listId,
                                                                    @Valid @RequestBody FeItemRequestDto feItemRequestDto,
                                                                    @CurrentUser User user) {

        ShoppingList shoppingList = shoppingListService.findWishListByIdOrThrow(user, listId);

        Product existedProductDb = productService.findProductBySkuAndCompanyOrThrow(feItemRequestDto.getOriginSku(), feItemRequestDto.getCompany());  // TODO missing groceryService.findByOriginSku if product is not found

        ShoppingListItem newItemReq = DtoFeMapper.mapToShoppingListItem(feItemRequestDto, existedProductDb);

        productService.findProductDuplicityByItem(shoppingList, newItemReq)
                .ifPresent(sameProductInUserCart -> {
                    throw new ResponseStatusException(HttpStatus.BAD_REQUEST, // TODO change error code (BAD_REQUEST ?)
                            "Product (name: " + newItemReq.getProduct().getName() +
                                    ", originSku:" + newItemReq.getProduct().getOriginSku() + ") is already in your cart.");
                });

        shoppingList.getShoppingListItems().add(newItemReq);
        shoppingListItemRepository.save(newItemReq);
        shoppingListRepository.save(shoppingList);
        return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(newItemReq), HttpStatus.CREATED);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PutMapping("/{listId}/items/{itemId}")
    public ResponseEntity<FeItemResponseDto> updateShoppingListItem(@PathVariable Long listId, @PathVariable Long itemId,
                                                                    @Valid @RequestBody FeItemRequestDto feItemRequestDto,
                                                                    @CurrentUser User user) {

        ShoppingList shoppingList = shoppingListService.findWishListByIdOrThrow(user, listId);
        ShoppingListItem existingItemToEdit = findShoppingListItemByIdOrThrow(shoppingList, itemId);

        Product existedProductDb = productService.findProductBySkuAndCompanyOrThrow(feItemRequestDto.getOriginSku(), feItemRequestDto.getCompany());

        ShoppingListItem updatedItemReq = DtoFeMapper.mapToShoppingListItem(feItemRequestDto, existedProductDb);
        updatedItemReq.setId(existingItemToEdit.getId());

//        updatedItemReq.getProduct().setId(existedProductDb.getId()); // TODO pointless? The product was already set up by mapper

        shoppingListItemRepository.save(updatedItemReq);
        shoppingListRepository.save(shoppingList);
        return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(updatedItemReq), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @DeleteMapping("/{listId}/items/{itemId}")
    public ResponseEntity<Void> deleteShoppingListItemById(@PathVariable Long listId, @PathVariable Long itemId, @CurrentUser User user) {

        ShoppingList shoppingList = shoppingListService.findWishListByIdOrThrow(user, listId);
        ShoppingListItem existingItem = findShoppingListItemByIdOrThrow(shoppingList, itemId);

        shoppingList.getShoppingListItems().remove(existingItem);
        shoppingListItemRepository.delete(existingItem);

        shoppingListRepository.save(shoppingList); // TODO not mandatory?
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @DeleteMapping("/{listId}/items")
    public ResponseEntity<Void> deleteAllShoppingListItems(@PathVariable Long listId, @CurrentUser User user) {

        ShoppingList shoppingList = shoppingListService.findWishListByIdOrThrow(user, listId);
        shoppingListItemRepository.deleteAll(shoppingList.getShoppingListItems());

        shoppingList.getShoppingListItems().clear();
        shoppingListRepository.save(shoppingList);

        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    private ShoppingListItem findShoppingListItemByIdOrThrow(ShoppingList shoppingList, Long itemId) {
        return shoppingList.getShoppingListItems().stream()
                .filter(item -> Objects.equals(item.getId(), itemId))
                .findFirst()
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.NOT_FOUND,
                        "ShoppingListItem NOT FOUND"));
    }
}