package cz.gottlieb.groceryAggregator.controllers;

import cz.gottlieb.groceryAggregator.configs.ConfigProperties;
import cz.gottlieb.groceryAggregator.persistence.model.Product;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.persistence.repository.ProductRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.UserRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.WatchDogRepository;
import cz.gottlieb.groceryAggregator.services.EmailService;
import cz.gottlieb.groceryAggregator.services.connectors.GroceryService;
import cz.gottlieb.groceryAggregator.utils.Constants;
import cz.gottlieb.groceryAggregator.utils.GroceryEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.Environment;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.HttpStatusCodeException;

import java.time.LocalDateTime;
import java.util.*;

@Slf4j
@Transactional
@RestController
@RequestMapping("/technical")
public class TechnicalController {

    private final ConfigProperties config;
    private final UserRepository userRepository;
    private final WatchDogRepository watchDogRepository;
    private final ProductRepository productRepository;
    private final Map<String, GroceryService> groceryServiceMap;
    private final EmailService emailService;
    private final Environment environment;


    public TechnicalController(ConfigProperties config,
                               WatchDogRepository watchDogRepository,
                               ProductRepository productRepository,
                               Map<String, GroceryService> groceryServiceMap,
                               EmailService emailService,
                               UserRepository userRepository,
                               Environment environment) {
        this.config = config;
        this.userRepository = userRepository;
        this.watchDogRepository = watchDogRepository;
        this.productRepository = productRepository;
        this.groceryServiceMap = groceryServiceMap;
        this.emailService = emailService;
        this.environment = environment;
    }

    @GetMapping("/healthCheck")
    public ResponseEntity<?> healthCheck() {
        log.info(">>> healthCheck");

        Map<String, String> map = new LinkedHashMap<>();
        map.put("name", config.getName());
        map.put("version", config.getVersion());

        log.info(map.toString());
        return new ResponseEntity<>(map, HttpStatus.OK);
    }

    @PostMapping("/run/watchdog")
    public ResponseEntity<Void> runWatchDogScan() {
        log.info(">>> runWatchDogScan");
        // No authentication & authorization

        Map<User, Set<Product>> selectedProducts = new HashMap<>();

        try {
            watchDogRepository.findAll().forEach(watchDog -> {
                Product existingProduct = watchDog.getProduct();
                GroceryEnum companyEnum = existingProduct.getCompany().getName();

                GroceryService groceryService = groceryServiceMap.get(companyEnum.name().toLowerCase() + "ServiceImpl"); // eg. rohlikServiceImpl

                // check lastUpdate
                Optional<Product> optProd = productRepository.findByOriginSkuAndCompany_Name(existingProduct.getOriginSku(), companyEnum);

                Product productResponse;
                if (optProd.isPresent() && optProd.get().getUpdateAt().isAfter(LocalDateTime.now().minusHours(24))) {
                    productResponse = optProd.get();
                } else {
                    productResponse = groceryService.findProductByNameAndSkuOrThrow(existingProduct.getName(), existingProduct.getOriginSku(), null);
                }

                productRepository.findByOriginSkuAndCompany_Name(existingProduct.getOriginSku(), companyEnum)
                        .ifPresent(productEntityDb -> productResponse.setId(productEntityDb.getId()));   // Product update

                productRepository.save(productResponse);

                if (watchDog.getPriceAlarm().compareTo(productResponse.getPriceFull()) >= 0) {
                    userRepository.findByWatchDogs_Product(productResponse)
                            .forEach(user -> {
                                if (selectedProducts.get(user) == null) {
                                    Set<Product> products = new HashSet<>();
                                    products.add(productResponse);
                                    selectedProducts.put(user, products);
                                } else {
                                    selectedProducts.get(user).add(productResponse);
                                }
                            });
                }
            });

            selectedProducts.forEach((user, productSet) -> {
                StringBuilder emailBody = new StringBuilder();

                for (Product product : productSet) {
                    emailBody.append("PRODUCT: ").append(product.getName())
                            .append("\n\t URL: ").append(product.getBaseUrl())
                            .append("\n\t PRICE (current): ").append(product.getPriceFull())
                            .append("\n\n");
                }

                sendEmail(user, emailBody);
            });

        } catch (HttpStatusCodeException exception) {
            return new ResponseEntity<>(null, exception.getStatusCode());
        }

        return new ResponseEntity<>(HttpStatus.OK);
    }

    private void sendEmail(User user, StringBuilder emailBody) {
        log.info("SENDING EMAIL TO: " + user.getEmail());
        log.info("SENDING EMAIL WITH BODY: ");
        log.info(emailBody.toString());

        if (Arrays.asList(environment.getActiveProfiles()).contains("production")) {
            // TODO change it !!!
            emailService.sendSimpleEmail("petr.sklenicka01@seznam.cz", Constants.EMAIL_SUBJECT, emailBody.toString());
        } else {
            // MOCK
            emailService.sendSimpleEmail(user.getEmail(), Constants.EMAIL_SUBJECT, emailBody.toString());
        }
    }
}
