package cz.gottlieb.groceryAggregator.controllers;

import cz.gottlieb.groceryAggregator.annotation.CurrentUser;
import cz.gottlieb.groceryAggregator.dto.fe.FeWatchDogDto;
import cz.gottlieb.groceryAggregator.dto.mapper.DtoFeMapper;
import cz.gottlieb.groceryAggregator.persistence.model.Product;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.persistence.model.WatchDog;
import cz.gottlieb.groceryAggregator.persistence.repository.WatchDogRepository;
import cz.gottlieb.groceryAggregator.services.ProductService;
import cz.gottlieb.groceryAggregator.services.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import javax.validation.Valid;
import java.util.Comparator;
import java.util.List;

@Slf4j
@Transactional
@RestController
@PreAuthorize("hasRole('user') and hasAuthority('SCOPE_read')")
@RequestMapping("/watchdog")
public class WatchDogController {

    private final UserService userService;
    private final WatchDogRepository watchDogRepository;
    private final ProductService productService;

    public WatchDogController(UserService userService,
                              WatchDogRepository watchDogRepository,
                              ProductService productService) {
        this.userService = userService;
        this.watchDogRepository = watchDogRepository;
        this.productService = productService;
    }

    @GetMapping
    public ResponseEntity<List<FeWatchDogDto>> getAllWatchDogs(@CurrentUser User user) {
        log.info(">>> getAllWatchDogs");

        List<WatchDog> watchDogs = user.getWatchDogs().stream()
                .sorted(Comparator.comparing(WatchDog::getId)).toList();

        return new ResponseEntity<>(DtoFeMapper.mapToFeWatchDogDto(watchDogs), HttpStatus.OK);
    }

    @GetMapping("/{id}")
    public ResponseEntity<FeWatchDogDto> getWatchDogById(@PathVariable Long id, @CurrentUser User user) {
        log.info(">>> getWatchDogById");

        WatchDog existingUserWatchDog = findWatchDogByIdOrThrow(user, id);

        return new ResponseEntity<>(DtoFeMapper.mapToFeWatchDogDto(existingUserWatchDog), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PostMapping
    public ResponseEntity<FeWatchDogDto> createWatchDog(@Valid @RequestBody FeWatchDogDto watchDogArg, @CurrentUser User user) {
        log.info(">>> createWatchDog");

        Product productEntityDb = productService.findProductBySkuAndCompanyOrThrow(watchDogArg.getOriginSku(), watchDogArg.getCompany());

        WatchDog newWatchDog = DtoFeMapper.mapToWatchDog(watchDogArg, productEntityDb);
        newWatchDog.setId(null); // Ensure that the ID is not set in the request body // TODO check everywhere

        user.getWatchDogs().add(newWatchDog);
        watchDogRepository.save(newWatchDog);
        return new ResponseEntity<>(DtoFeMapper.mapToFeWatchDogDto(newWatchDog), HttpStatus.CREATED);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @PutMapping("/{id}")
    public ResponseEntity<FeWatchDogDto> updateWatchDog(@PathVariable Long id, @Valid @RequestBody FeWatchDogDto watchDogArg,
                                                        @CurrentUser User user) {
        log.info(">>> updateWatchDog");

        Product productEntityDb = productService.findProductBySkuAndCompanyOrThrow(watchDogArg.getOriginSku(), watchDogArg.getCompany());

        WatchDog existingUserWatchDog = findWatchDogByIdOrThrow(user, id);

        WatchDog updatedWatchDog = DtoFeMapper.mapToWatchDog(watchDogArg, productEntityDb);
        updatedWatchDog.setId(existingUserWatchDog.getId());

        watchDogRepository.save(updatedWatchDog);
        return new ResponseEntity<>(DtoFeMapper.mapToFeWatchDogDto(existingUserWatchDog), HttpStatus.OK);
    }

    @PreAuthorize("hasAuthority('SCOPE_write')")
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteWatchDog(@PathVariable Long id, @CurrentUser User user) {
        log.info(">>> deleteWatchDog");

        WatchDog existingUserWatchDog = findWatchDogByIdOrThrow(user, id);

        user.getWatchDogs().remove(existingUserWatchDog);
        userService.save(user);

        watchDogRepository.deleteById(id);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    private WatchDog findWatchDogByIdOrThrow(User user, Long id) {
        return user.getWatchDogs().stream()
                .filter(w -> w.getId().equals(id))
                .findFirst()
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.NOT_FOUND,
                        "WatchDog (id: " + id + ") NOT found"));
    }

}

