package cz.gottlieb.groceryAggregator.controllers.connectors;

import com.fasterxml.jackson.core.JsonProcessingException;
import cz.gottlieb.groceryAggregator.annotation.CurrentUser;
import cz.gottlieb.groceryAggregator.dto.fe.FeItemRequestDto;
import cz.gottlieb.groceryAggregator.dto.fe.FeItemResponseDto;
import cz.gottlieb.groceryAggregator.dto.fe.FeProductResponseDto;
import cz.gottlieb.groceryAggregator.dto.mapper.DtoFeMapper;
import cz.gottlieb.groceryAggregator.persistence.model.Product;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingListItem;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.services.connectors.impl.RohlikServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.util.Pair;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.HttpStatusCodeException;
import org.springframework.web.server.ResponseStatusException;

import javax.validation.Valid;
import java.util.List;

@Slf4j
@Transactional
@RestController
@PreAuthorize("hasRole('user') and hasAuthority('SCOPE_read')")
@RequestMapping("/api/rohlik")
public class RohlikController implements RohlikRestApi {
    private final RohlikServiceImpl rohlikService;

    public RohlikController(RohlikServiceImpl rohlikService) {
        this.rohlikService = rohlikService;
    }

    @Override
    @GetMapping("/findByName")
    public ResponseEntity<List<FeProductResponseDto>> findByName(@RequestParam String name) {
        log.info(">>> rohlikFindByName");

        List<Product> products = List.of();
        Pair<HttpStatus, List<Product>> result;

        try {
            result = rohlikService.findProducts(name);
            products = result.getSecond();
        } catch (HttpStatusCodeException exception) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeProductResponseDto(products), exception.getStatusCode());
        } catch (JsonProcessingException e) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeProductResponseDto(products), HttpStatus.INTERNAL_SERVER_ERROR);
        }

        return new ResponseEntity<>(DtoFeMapper.mapToFeProductResponseDto(products), result.getFirst());
    }

    @Override
    @GetMapping("/cart")
    public ResponseEntity<List<FeItemResponseDto>> getCart(@CurrentUser User user) {
        log.info(">>> rohlikGetCart");
        tryToLogin(user);

        List<ShoppingListItem> rohlikItems = List.of();
        Pair<HttpStatus, List<ShoppingListItem>> result;
        try {
            result = rohlikService.loadCart();
            rohlikItems = result.getSecond();

            /**
             * We will store NEITHER products NOR items
             *
             ShoppingList selectedShoppingList = shoppingListService.findDefaultCartOrThrow(user);
             shoppingListService.saveOrUpdate(selectedShoppingList, rohlikItems);
             */
        } catch (HttpStatusCodeException exception) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(rohlikItems), exception.getStatusCode());
        } catch (JsonProcessingException e) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(rohlikItems), HttpStatus.INTERNAL_SERVER_ERROR);
        }

        return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(rohlikItems), result.getFirst());
    }

    @Override
    @PostMapping("/cart")
    public ResponseEntity<List<FeItemResponseDto>> addToCart(@Valid @RequestBody FeItemRequestDto feItemRequestDto, @CurrentUser User user) {
        log.info(">>> rohlikAddToCart");
        validate(feItemRequestDto);

        log.info("originSku: " + feItemRequestDto.getOriginSku());
        log.info("quantity: " + feItemRequestDto.getQuantity());
        log.info("originItemId (empty!?): " + feItemRequestDto.getOriginItemId());

        tryToLogin(user);

        ShoppingListItem feItemRequest = DtoFeMapper.mapToShoppingListItem(feItemRequestDto, new Product());

        List<ShoppingListItem> rohlikItems = List.of();
        Pair<HttpStatus, List<ShoppingListItem>> result;
        try {
            result = rohlikService.addToCart(feItemRequest);
            rohlikItems = result.getSecond();

            /**
             * We will store NEITHER products NOR items
             *
             ShoppingList selectedShoppingList = shoppingListService.findDefaultCartOrThrow(user);
             shoppingListService.saveOrUpdate(selectedShoppingList, rohlikItems);
             */
        } catch (HttpStatusCodeException exception) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(rohlikItems), exception.getStatusCode());
        } catch (JsonProcessingException e) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(rohlikItems), HttpStatus.INTERNAL_SERVER_ERROR);
        }

        return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(rohlikItems), result.getFirst());
    }

    @Override
    @PutMapping("/cart")
    public ResponseEntity<List<FeItemResponseDto>> changeCart(@Valid @RequestBody FeItemRequestDto feItemRequestDto, @CurrentUser User user) {
        log.info(">>> rohlikChangeCart");
        validate(feItemRequestDto);

        log.info("originSku: " + feItemRequestDto.getOriginSku());
        log.info("quantity: " + feItemRequestDto.getQuantity());
        log.info("originItemId: " + feItemRequestDto.getOriginItemId());

        tryToLogin(user);

        ShoppingListItem feItemRequest = DtoFeMapper.mapToShoppingListItem(feItemRequestDto, new Product());

        List<ShoppingListItem> rohlikItems = List.of();
        Pair<HttpStatus, List<ShoppingListItem>> result;
        try {
            if (feItemRequestDto.getQuantity() >= 1) {
                result = rohlikService.changeCart(feItemRequest);
                rohlikItems = result.getSecond();
            } else {
                result = rohlikService.removeFromCart(feItemRequest);
                rohlikItems = result.getSecond();
            }

            /**
             * We will store NEITHER products NOR items
             *
             ShoppingList selectedShoppingList = shoppingListService.findDefaultCartOrThrow(user);
             shoppingListService.saveOrUpdate(selectedShoppingList, rohlikItems);
             */
        } catch (HttpStatusCodeException exception) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(rohlikItems), exception.getStatusCode());
        } catch (JsonProcessingException e) {
            return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(rohlikItems), HttpStatus.INTERNAL_SERVER_ERROR);
        }

        return new ResponseEntity<>(DtoFeMapper.mapToFeItemResponseDto(rohlikItems), result.getFirst());
    }

    @Override
    @DeleteMapping("/cart/all")
    public ResponseEntity<Boolean> clearCart(@CurrentUser User user) {
        log.info(">>> rohlikClearCart");
        tryToLogin(user);

        HttpStatus status = rohlikService.clearCart();
        return new ResponseEntity<>(status == HttpStatus.OK, status);
    }

    @Override
    @PostMapping("/login")
    public ResponseEntity<Boolean> login(@CurrentUser User user) {
        log.info(">>> rohlikLogin");

        HttpStatus status = rohlikService.loginOrThrow(user);

        return new ResponseEntity<>(status == HttpStatus.OK, status);
    }

    @Override
    @PostMapping("/logout")
    public ResponseEntity<Boolean> logout() {
        log.info(">>> rohlikLogout");

        HttpStatus status = rohlikService.logout();

        return new ResponseEntity<>(status == HttpStatus.OK, status);
    }

    private void validate(FeItemRequestDto feItemRequestDto) {
        if (feItemRequestDto.getQuantity() == null) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, ""); // TODO
        }
    }

    private HttpStatus tryToLogin(User user) {
        HttpStatus status = null;
        try {
            status = rohlikService.loginOrThrow(user);
        } catch (ResponseStatusException e) {
        }
        return status;
    }
}
