package cz.gottlieb.groceryAggregator.services.connectors.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import cz.gottlieb.groceryAggregator.configs.connectors.BillaConfig;
import cz.gottlieb.groceryAggregator.dto.billa.BillaCredentialDto;
import cz.gottlieb.groceryAggregator.dto.billa.BillaItemRequestDto;
import cz.gottlieb.groceryAggregator.dto.billa.BillaItemResponseDto;
import cz.gottlieb.groceryAggregator.dto.billa.BillaProductResponseDto;
import cz.gottlieb.groceryAggregator.dto.mapper.DtoBillaMapper;
import cz.gottlieb.groceryAggregator.persistence.model.*;
import cz.gottlieb.groceryAggregator.persistence.repository.ProductRepository;
import cz.gottlieb.groceryAggregator.services.CompanyService;
import cz.gottlieb.groceryAggregator.services.CredentialsService;
import cz.gottlieb.groceryAggregator.utils.GroceryEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.util.Pair;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.context.annotation.SessionScope;

import javax.annotation.PostConstruct;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

@Slf4j
@SessionScope
@Service
public class BillaServiceImpl extends GroceryBaseServiceImpl {
    public static String serverApiUrl;
    //
    private final RestTemplate restTemplate;
    private final ObjectMapper jsonObjectMapper;
    private final CompanyService companyService;
    private final BillaConfig config;
    private final ProductRepository productRepository;
    private final CredentialsService credentialsService;
    //
    private LocalDateTime loggedInTime = null;
    private Company billaDbEntity;

    public BillaServiceImpl(@Qualifier("billaRestTemplate") RestTemplate restTemplate,
                            @Qualifier("myJsonObjectMapper") ObjectMapper jsonObjectMapper,
                            CompanyService companyService,
                            CredentialsService credentialsService,
                            BillaConfig config,
                            ProductRepository productRepository) {
        super();

        this.restTemplate = restTemplate;
        this.jsonObjectMapper = jsonObjectMapper;
        this.companyService = companyService;
        this.config = config;
        this.productRepository = productRepository;
        this.credentialsService = credentialsService;
    }

    @PostConstruct
    private void init() {
        log.info(">>> init");

        setUpConfig();
        initCompany();
    }

    private void setUpConfig() {
        log.info(">>> setUpConfig");

        config.setup();
        serverApiUrl = config.getServerApiUrl();
    }

    private void initCompany() {
        log.info(">>> initCompany");

        billaDbEntity = companyService.setUpCompany(GroceryEnum.BILLA, config);
    }

    @Override
    public Pair<HttpStatus, List<Product>> findProducts(String name) throws JsonProcessingException {
        log.info(">>> findProducts");
        log.info("param name:" + name);

        final String url = serverApiUrl + "/products/search/" + name + "?storeId=82-189";

        HttpEntity httpEntity = new HttpEntity(null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.GET, httpEntity, String.class);

        List<BillaProductResponseDto> productDtos = parseProducts(response);

/*      for (ProductDto prod : productDtos) {
            log.info(prod.toString());
        }*/

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());

        try {
            log.info(jsonObjectMapper.writeValueAsString(productDtos));
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }

        return Pair.of(response.getStatusCode(), DtoBillaMapper.mapToProduct(productDtos, billaDbEntity));
    }

    @Override
    public Pair<HttpStatus, List<ShoppingListItem>> loadCart() throws JsonProcessingException {
        log.info(">>> loadCart");

        final String url = serverApiUrl + "/carts";

        HttpEntity httpEntity = new HttpEntity(null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.GET, httpEntity, String.class);

        List<BillaItemResponseDto> itemDtos = parseItems(response);

/*        for (ItemDto item : itemDtos) {
            log.info(item.toString());
        }*/

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return Pair.of(response.getStatusCode(), DtoBillaMapper.mapToShoppingListItem(itemDtos, billaDbEntity));
    }

    @Override
    public HttpStatus clearCart() {
        log.info(">>> clearCart");

        final String url = serverApiUrl + "/carts/items/all";

        HttpEntity httpEntity = new HttpEntity(null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.DELETE, httpEntity, String.class);

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return response.getStatusCode();
    }

    @Override
    public GroceryEnum getType() {
        return GroceryEnum.BILLA;
    }

    @Override
    public Pair<HttpStatus, List<ShoppingListItem>> addToCart(ShoppingListItem shoppingListItem) throws JsonProcessingException {
        log.info(">>> addToCart");

        final String url = serverApiUrl + "/carts/items";

        HttpEntity<BillaItemRequestDto> httpEntity = new HttpEntity<>(DtoBillaMapper.mapToBillaCartItemPostDto(shoppingListItem), null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, httpEntity, String.class);

        List<BillaItemResponseDto> itemDtos = parseItems(response);

/*        for (BillaItemResponseDto item : itemDtos) {
            log.info(item.toString());
        }*/

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return Pair.of(response.getStatusCode(), DtoBillaMapper.mapToShoppingListItem(itemDtos, billaDbEntity));
    }

    @Override
    public Pair<HttpStatus, List<ShoppingListItem>> changeCart(ShoppingListItem shoppingListItem) throws JsonProcessingException {
        log.info(">>> changeCart");

        final String url = serverApiUrl + "/carts/items";

        HttpEntity<BillaItemRequestDto> httpEntity = new HttpEntity<>(DtoBillaMapper.mapToBillaCartItemPutDto(shoppingListItem), null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, httpEntity, String.class);

        List<BillaItemResponseDto> itemDtos = parseItems(response);

/*        for (ItemDto item : itemDtos) {
            log.info(item.toString());
        }*/

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return Pair.of(response.getStatusCode(), DtoBillaMapper.mapToShoppingListItem(itemDtos, billaDbEntity));
    }

    @Override
    public Pair<HttpStatus, List<ShoppingListItem>> removeFromCart(ShoppingListItem shoppingListItem) throws JsonProcessingException {
        log.info(">>> removeFromCart");

        shoppingListItem.setQuantity(0);

        return changeCart(shoppingListItem);
    }

    @Override
    public HttpStatus login(GroceryCredential cred) {
        log.info(">>> login");

        if (loggedInTime != null && loggedInTime.isAfter(LocalDateTime.now().minusMinutes(config.getLogTimeMinutes()))) {
            log.info("skipping login:");
            log.info("\t\tloggedInTime:" + loggedInTime.toString());
            return HttpStatus.OK;
        }
        initToken();

        final String url = serverApiUrl + "/auth/login";

        HttpEntity<BillaCredentialDto> httpEntity = new HttpEntity<>(DtoBillaMapper.mapToBillaCredentialDto(cred), null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, httpEntity, String.class);

        HttpStatus loadCartStatus = HttpStatus.OK;
        try {
            loadCartStatus = loadCart().getFirst();
        } catch (JsonProcessingException e) {
        }

        if (response.getStatusCode().is2xxSuccessful() && loadCartStatus.is2xxSuccessful()) {
            loggedInTime = LocalDateTime.now();
        } else {
            loggedInTime = null;
        }

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return response.getStatusCode();
    }

    public HttpStatus loginOrThrow(User user) {
        GroceryCredential credentials = credentialsService.findCredentialsByGroceryNameOrThrow(user, GroceryEnum.BILLA);
        return login(credentials);
    }

    @Override
    public HttpStatus logout() {
        log.info(">>> logout");
        loggedInTime = null;

        final String url = serverApiUrl + "/auth/logout";

        HttpEntity httpEntity = new HttpEntity(null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, httpEntity, String.class);

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return response.getStatusCode();
    }

    private void initToken() {
        final String url = serverApiUrl + "/csrf";

        HttpEntity<BillaCredentialDto> httpEntity = new HttpEntity<>(new HttpHeaders());

        restTemplate.exchange(url, HttpMethod.GET, httpEntity, String.class);
    }

    private List<BillaProductResponseDto> parseProducts(ResponseEntity<String> response) throws JsonProcessingException {
        log.info(">>> parseFoundedProducts");

        List<BillaProductResponseDto> productDtos;

        JsonNode jsonNode = jsonObjectMapper.readTree(response.getBody());
        JsonNode productListNode = jsonNode.at("/results");

        productDtos = jsonObjectMapper.convertValue(productListNode, new TypeReference<List<BillaProductResponseDto>>() {
        });
        return productDtos;

    }

    private List<BillaItemResponseDto> parseItems(ResponseEntity<String> response) throws JsonProcessingException {
        log.info(">>> parseCartItems");

        List<BillaItemResponseDto> itemDtoList;

        JsonNode jsonNode = jsonObjectMapper.readTree(response.getBody());
        JsonNode productListNode = jsonNode.at("/productLineItems");

        itemDtoList = jsonObjectMapper.convertValue(productListNode, new TypeReference<List<BillaItemResponseDto>>() {
        });

        if (itemDtoList == null) {
            return Collections.emptyList();
        }
        return itemDtoList;
    }


}
