package cz.gottlieb.groceryAggregator.services.connectors.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import cz.gottlieb.groceryAggregator.configs.connectors.RohlikConfig;
import cz.gottlieb.groceryAggregator.dto.mapper.DtoRohlikMapper;
import cz.gottlieb.groceryAggregator.dto.rohlik.*;
import cz.gottlieb.groceryAggregator.persistence.model.*;
import cz.gottlieb.groceryAggregator.persistence.repository.ProductRepository;
import cz.gottlieb.groceryAggregator.services.CompanyService;
import cz.gottlieb.groceryAggregator.services.CredentialsService;
import cz.gottlieb.groceryAggregator.utils.GroceryEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.util.Pair;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.context.annotation.SessionScope;

import javax.annotation.PostConstruct;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;

@Slf4j
@SessionScope
@Service
public class RohlikServiceImpl extends GroceryBaseServiceImpl {
    public static String serverApiUrl;
    //
    private final RestTemplate restTemplate;
    private final ObjectMapper jsonObjectMapper;
    private final RohlikConfig config;
    private final CompanyService companyService;
    private final ProductRepository productRepository;
    private final CredentialsService credentialsService;
    //
    private LocalDateTime loggedInTime = null;
    private Company rohlikDbEntity;

    public RohlikServiceImpl(@Qualifier("rohlikRestTemplate") RestTemplate restTemplate,
                             @Qualifier("myJsonObjectMapper") ObjectMapper jsonObjectMapper,
                             RohlikConfig config,
                             CompanyService companyService,
                             CredentialsService credentialsService,
                             ProductRepository productRepository) {
        super();

        this.restTemplate = restTemplate;
        this.jsonObjectMapper = jsonObjectMapper;
        this.config = config;
        this.companyService = companyService;
        this.productRepository = productRepository;
        this.credentialsService = credentialsService;
    }

    @PostConstruct
    private void init() {
        log.info(">>> init");

        setUpConfig();
        initCompany();
    }

    private void setUpConfig() {
        log.info(">>> setUpConfig");

        config.setup();
        serverApiUrl = config.getServerApiUrl();
    }

    private void initCompany() {
        log.info(">>> initCompany");

        rohlikDbEntity = companyService.setUpCompany(GroceryEnum.ROHLIK, config);
    }

    @Override
    public Pair<HttpStatus, List<Product>> findProducts(String name) throws JsonProcessingException {
        log.info(">>> findProducts");
        log.info("param name:" + name);

        final String url = serverApiUrl + "/services/frontend-service/search-metadata?search=" + name
                + "&companyId=1&limit=25&filterData={}&canCorrect=true";

        HttpEntity httpEntity = new HttpEntity(null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.GET, httpEntity, String.class);

        List<RohlikProductResponseDto> productDtos = parseProducts(response);

/*      for (ProductDto prod : productDtos) {
            log.info(prod.toString());
        }*/

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());

        try {
            log.info(jsonObjectMapper.writeValueAsString(productDtos));
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }

        return Pair.of(response.getStatusCode(), DtoRohlikMapper.mapToProduct(productDtos, rohlikDbEntity));
    }

    @Override
    public Pair<HttpStatus, List<ShoppingListItem>> loadCart() throws JsonProcessingException {
        log.info(">>> loadCart");

        final String url = serverApiUrl + "/services/frontend-service/v2/cart";

        HttpEntity httpEntity = new HttpEntity(null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.GET, httpEntity, String.class);

        List<RohlikItemResponseDto> itemDtos = parseItems(response);

/*        for (ItemDto item : itemDtos) {
            log.info(item.toString());
        }*/

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return Pair.of(response.getStatusCode(), DtoRohlikMapper.mapToShoppingListItem(itemDtos, rohlikDbEntity));
    }

    @Override
    public HttpStatus clearCart() {
        log.info(">>> clearCart");

        final String url = serverApiUrl + "/services/frontend-service/v2/cart?clear=true";

        HttpEntity httpEntity = new HttpEntity(null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.DELETE, httpEntity, String.class);

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return response.getStatusCode();
    }

    @Override
    public GroceryEnum getType() {
        return GroceryEnum.ROHLIK;
    }

    @Override
    public Pair<HttpStatus, List<ShoppingListItem>> addToCart(ShoppingListItem shoppingListItem) throws JsonProcessingException {
        log.info(">>> addToCart");

        final String url = serverApiUrl + "/services/frontend-service/v2/cart";

        HttpEntity<RohlikItemPostDto> httpEntity = new HttpEntity<>(DtoRohlikMapper.mapToRohlikCartItemPostDto(shoppingListItem), null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, httpEntity, String.class);

        List<RohlikItemResponseDto> itemDtos = parseItems(response);

/*        for (RohlikItemDto item : itemDtos) {
            log.info(item.toString());
        }*/


        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return Pair.of(response.getStatusCode(), DtoRohlikMapper.mapToShoppingListItem(itemDtos, rohlikDbEntity));
    }

    @Override
    public Pair<HttpStatus, List<ShoppingListItem>> changeCart(ShoppingListItem shoppingListItem) throws JsonProcessingException {
        log.info(">>> changeCart");

        final String url = serverApiUrl + "/services/frontend-service/v2/cart?orderFieldId=" + shoppingListItem.getOriginItemId();

        HttpEntity<RohlikItemPutDto> httpEntity = new HttpEntity<>(DtoRohlikMapper.mapToRohlikCartItemPutDto(shoppingListItem), null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.PUT, httpEntity, String.class);

        List<RohlikItemResponseDto> itemDtos = parseItems(response);

/*        for (ItemDto item : itemDtos) {
            log.info(item.toString());
        }*/

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return Pair.of(response.getStatusCode(), DtoRohlikMapper.mapToShoppingListItem(itemDtos, rohlikDbEntity));
    }

    @Override
    public Pair<HttpStatus, List<ShoppingListItem>> removeFromCart(ShoppingListItem shoppingListItem) throws JsonProcessingException {
        log.info(">>> removeFromCart");

        final String url = serverApiUrl + "/services/frontend-service/v2/cart?orderFieldId=" + shoppingListItem.getOriginItemId();

        HttpEntity httpEntity = new HttpEntity(null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.DELETE, httpEntity, String.class);

        List<RohlikItemResponseDto> itemDtos = parseItems(response);

/*        for (ItemDto item : itemDtos) {
            log.info(item.toString());
        }*/

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return Pair.of(response.getStatusCode(), DtoRohlikMapper.mapToShoppingListItem(itemDtos, rohlikDbEntity));
    }

    @Override
    public HttpStatus login(GroceryCredential cred) {
        log.info(">>> login");

        if (loggedInTime != null && loggedInTime.isAfter(LocalDateTime.now().minusMinutes(config.getLogTimeMinutes()))) {
            log.info("skipping login:");
            log.info("\t\tloggedInTime:" + loggedInTime.toString());
            return HttpStatus.OK;
        }

        final String url = serverApiUrl + "/services/frontend-service/login";

        HttpEntity<RohlikCredentialDto> httpEntity = new HttpEntity<>(DtoRohlikMapper.mapToRohlikCredentialDto(cred), null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, httpEntity, String.class);

        if (response.getStatusCode().is2xxSuccessful()) {
            loggedInTime = LocalDateTime.now();
        } else {
            loggedInTime = null;
        }

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return response.getStatusCode();
    }

    public HttpStatus loginOrThrow(User user) {
        GroceryCredential credentials = credentialsService.findCredentialsByGroceryNameOrThrow(user, GroceryEnum.ROHLIK);
        return login(credentials);
    }

    @Override
    public HttpStatus logout() {
        log.info(">>> logout");
        loggedInTime = null;

        final String url = serverApiUrl + "/services/frontend-service/logout";

        HttpEntity httpEntity = new HttpEntity(null);

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, httpEntity, String.class);

        log.info("response.getStatusCode: " + response.getStatusCode());
//        log.info("response.getBody: " + response.getBody());
        return response.getStatusCode();
    }

    private List<RohlikProductResponseDto> parseProducts(ResponseEntity<String> response) throws JsonProcessingException {
        log.info(">>> parseFoundedProducts");

        List<RohlikProductResponseDto> productDtos;

        JsonNode jsonNode = jsonObjectMapper.readTree(response.getBody());
        JsonNode productListNode = jsonNode.at("/data/productList");

        productDtos = jsonObjectMapper.convertValue(productListNode, new TypeReference<List<RohlikProductResponseDto>>() {
        });
        return productDtos;
    }

    private List<RohlikItemResponseDto> parseItems(ResponseEntity<String> response) throws JsonProcessingException {
        log.info(">>> parseCartItems");

        Map<String, RohlikItemResponseDto> itemDtoMap;

        JsonNode jsonNode = jsonObjectMapper.readTree(response.getBody());
        JsonNode productListNode = jsonNode.at("/data/items");

        itemDtoMap = jsonObjectMapper.convertValue(productListNode, new TypeReference<Map<String, RohlikItemResponseDto>>() {
        });

        if (itemDtoMap == null) {
            return Collections.emptyList();
        }
        return itemDtoMap.values().stream().toList();
    }

}
