package cz.gottlieb.groceryAggregator.services.impl;

import cz.gottlieb.groceryAggregator.persistence.model.ShoppingList;
import cz.gottlieb.groceryAggregator.persistence.model.ShoppingListItem;
import cz.gottlieb.groceryAggregator.persistence.model.User;
import cz.gottlieb.groceryAggregator.persistence.repository.CompanyRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.ProductRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.ShoppingListItemRepository;
import cz.gottlieb.groceryAggregator.persistence.repository.ShoppingListRepository;
import cz.gottlieb.groceryAggregator.services.ShoppingListService;
import cz.gottlieb.groceryAggregator.utils.ShoppingListEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

@Slf4j
@Service
public class ShoppingListServiceImpl implements ShoppingListService {

    private final ShoppingListRepository shoppingListRepository;
    private final ShoppingListItemRepository shoppingListItemRepository;
    private final ProductRepository productRepository;
    private final CompanyRepository companyRepository;

    public ShoppingListServiceImpl(ShoppingListRepository shoppingListRepository,
                                   ShoppingListItemRepository shoppingListItemRepository,
                                   ProductRepository productRepository,
                                   CompanyRepository companyRepository) {
        this.shoppingListRepository = shoppingListRepository;
        this.shoppingListItemRepository = shoppingListItemRepository;
        this.productRepository = productRepository;
        this.companyRepository = companyRepository;
    }

    public void saveOrUpdate(ShoppingList shoppingList, List<ShoppingListItem> items) {
        for (ShoppingListItem anItemReq : items) {
            Optional<ShoppingListItem> sameItemInUserList = shoppingList.getShoppingListItems().stream()
                    .filter(item -> Objects.equals(anItemReq.getOriginItemId(), item.getOriginItemId()))
                    .filter(item -> Objects.equals(anItemReq.getProduct().getCompany(), item.getProduct().getCompany())) // originItemId can be (in theory) the same in other companies
                    .findFirst();

            if (sameItemInUserList.isPresent()) {
                anItemReq.setId(sameItemInUserList.get().getId());  // ShoppingListItem update
            }

            productRepository.findByOriginSkuAndCompany_Name(anItemReq.getProduct().getOriginSku(),
                            anItemReq.getProduct().getCompany().getName())
                    .ifPresent(productEntityDb -> anItemReq.getProduct().setId(productEntityDb.getId()));   // Product update

            companyRepository.findByName(anItemReq.getProduct().getCompany().getName())
                    .ifPresentOrElse(
                            (companyEntityDb) ->
                                    anItemReq.getProduct().setCompany(companyEntityDb),  // Product update
                            () -> {
                                throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR,
                                        "Company (" + anItemReq.getProduct().getCompany().getName() + ") NOT found");
                            }
                    );

            productRepository.save(anItemReq.getProduct());
            shoppingList.getShoppingListItems().add(anItemReq);
            shoppingListItemRepository.save(anItemReq);
        }
        shoppingListRepository.save(shoppingList);
    }

    public ShoppingList findWishListByIdOrThrow(User user, Long id) {
        return user.getShoppingLists().stream()
                .filter(sList -> ShoppingListEnum.WISHLIST.equals(sList.getType()))
                .filter(sList -> sList.getId().equals(id))
                .findFirst()
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.NOT_FOUND,
                        "ShoppingList (Wishlist) with id " + id + " NOT FOUND"));
    }

    public List<ShoppingList> findAllWishLists(User user) {
        return user.getShoppingLists().stream()
                .filter(shoppingList -> ShoppingListEnum.WISHLIST.equals(shoppingList.getType()))
                .toList();
    }

    public ShoppingList findDefaultCartOrThrow(User user) {
        return user.getShoppingLists().stream()
                .filter(shoppingList -> ShoppingListEnum.DEFAULT_CART.equals(shoppingList.getType()))
                .findFirst()
                .orElseThrow(() -> new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR,
                        "ShoppingList NOT FOUND"));
    }

}
