import {Component, Input, OnInit} from '@angular/core';
import {FormBuilder, Validators} from "@angular/forms";
import {GroceryCredentialService} from "../../../services/rest/grocery-credential-service.service";
import {GroceryCredentialDto} from "../../../dto/GroceryCredentialDto";
import {CompanyEnum} from "../../../util/CompanyEnum";
import {HttpErrorResponse} from "@angular/common/http";
import {FuncsService} from "../../../services/funcs.service";

@Component({
  selector: 'app-grocery-credential-detail',
  templateUrl: './grocery-credential-detail.component.html',
  styleUrl: './grocery-credential-detail.component.scss'
})
export class GroceryCredentialDetailComponent implements OnInit {
  readonly CompanyEnum = CompanyEnum;
  readonly COMPANY_PATTERN: string = CompanyEnum.ROHLIK + '|' + CompanyEnum.BILLA;

  @Input()
  credential?: GroceryCredentialDto;

  @Input({required: true})
  shouldHide!: boolean;

  detailForm = this.fb.group({
    id: [null as number | null],
    company: ['' as CompanyEnum, {
      validators: [Validators.required, Validators.pattern(new RegExp(this.COMPANY_PATTERN, "i"))],
      nonNullable: true
    }],
    username: ['' as string, {validators: [Validators.required], nonNullable: true}],
    password: ['', {validators: [Validators.required], nonNullable: true}]
  });
  controlsForm = this.detailForm.controls;

  constructor(private credentialService: GroceryCredentialService,
              private fb: FormBuilder,
              private funcsService: FuncsService) {
  }

  ngOnInit(): void {
    this.loadCredential();
  }

  isSaveEnabled(): boolean {
    let formValid: boolean = this.detailForm.valid
      && this.controlsForm.company.value !== null
      && this.controlsForm.username.value !== ''
      && this.controlsForm.password.value !== '';

    let formChanged: boolean = this.detailForm.dirty;

    return formValid && formChanged;
  }

  isDeleteEnabled(): boolean {
    return this.controlsForm.id.value !== null;
  }

  saveOrUpdate(): void {
    if (this.controlsForm.id.value === null) {
      this.save();
    } else {
      this.update();
    }
  }

  delete(): void {
    if (!this.isDeleteEnabled()) {
      return;
    }
    this.credentialService.deleteCredential(this.controlsForm.id.value!).subscribe(value => {
      this.funcsService.redirectTo();
    });
  }

  ///////////////////////////////////////////////////
  //<editor-fold desc="ngOnInit supporting methods">
  ///////////////////////////////////////////////////
  private loadCredential(): void {
    if (this.credential !== undefined) {
      this.controlsForm.id.setValue(this.credential.id);
      this.controlsForm.company.setValue(this.credential.company);
      this.controlsForm.username.setValue(this.credential.username);
      this.controlsForm.password.setValue(this.credential.password);
    }
  }

  ///////////////////////////////////////////////////
  //</editor-fold>
  ///////////////////////////////////////////////////

  private save(): void {
    let cred = this.fillCredential(null);

    this.credentialService.addCredential(cred).subscribe({
      next: value => {
        console.log(value); // TODO delete it
        this.funcsService.redirectTo();
      },
      error: err => {
        const tmp = new HttpErrorResponse(err);
        console.log(tmp.message); // TODO delete it?
        console.error(tmp);
      }
    });
  }

  private update(): void {
    let cred = this.fillCredential(this.controlsForm.id.value);

    this.credentialService.updateCredential(cred).subscribe(value => {
      console.log(value); // TODO delete it
      this.credential = value;
      this.detailForm.reset(this.detailForm.value);
      this.loadCredential();
    });
  }

  private fillCredential(idArg: number | null) {
    let cred: GroceryCredentialDto = {
      id: idArg,
      username: this.controlsForm.username.value!,
      password: this.controlsForm.password.value!,
      company: this.controlsForm.company.value!,
    };
    return cred;
  }
}
