import {Component, Input, OnDestroy, OnInit} from '@angular/core';
import {FormBuilder, Validators} from "@angular/forms";
import {CompanyEnum} from "../../../util/CompanyEnum";
import {HttpErrorResponse} from "@angular/common/http";
import {WatchDogDto} from "../../../dto/WatchDogDto";
import {WatchDogService} from "../../../services/rest/watch-dog.service";
import {ProductDto} from "../../../dto/ProductDto";
import {MessageService} from "../../../services/message.service";
import {FuncsService} from "../../../services/funcs.service";
import {AggregatedSearchService} from "../../../services/rest/aggregated-search.service";
import {takeWhile} from "rxjs";

@Component({
  selector: 'app-watch-dog-detail',
  templateUrl: './watch-dog-detail.component.html',
  styleUrl: './watch-dog-detail.component.scss'
})
export class WatchDogDetailComponent implements OnInit, OnDestroy {
  readonly CompanyEnum = CompanyEnum;
  readonly COMPANY_PATTERN: string = CompanyEnum.ROHLIK + '|' + CompanyEnum.BILLA;
  alive: boolean = true;

  @Input()
  dog?: WatchDogDto;

  selectedProduct?: ProductDto;

  detailForm = this.fb.group({
    id: [null as number | null],
    company: ['' as CompanyEnum, {
      validators: [Validators.required, Validators.pattern(new RegExp(this.COMPANY_PATTERN, "i"))],
      nonNullable: true
    }],
    originSku: ['' as string, {validators: [Validators.required], nonNullable: true}],
    productName: ['' as string, {validators: [Validators.required], nonNullable: true}],
    priceAlarm: [0, {validators: [Validators.required], nonNullable: true}]
  });
  controlsForm = this.detailForm.controls;

  constructor(private watchDogService: WatchDogService,
              private searchService: AggregatedSearchService,
              private messageService: MessageService,
              private fb: FormBuilder,
              private funcsService: FuncsService) {
  }

  ngOnInit(): void {
    this.loadWatchDog();
    this.disableControlls();
    this.bindToSelectedProduct();
  }

  ngOnDestroy() {
    this.alive = false;
  }

  isSaveEnabled(): boolean {
    let formValid: boolean = this.detailForm.valid
      && this.controlsForm.company.value !== null
      && this.controlsForm.productName.value !== ''
      && this.controlsForm.originSku.value !== ''
      && this.controlsForm.priceAlarm.value !== 0;

    let formChanged: boolean = this.detailForm.dirty;

    return formValid && formChanged;
  }

  isDeleteEnabled(): boolean {
    return this.controlsForm.id.value !== null;
  }

  saveOrUpdate(): void {
    if (this.controlsForm.id.value === null) {
      this.save();
    } else {
      this.update();
    }
  }

  delete(): void {
    if (!this.isDeleteEnabled()) {
      return;
    }
    this.watchDogService.deleteWatchDog(this.controlsForm.id.value!).subscribe(value => {
      this.funcsService.redirectTo();
    });
  }

  ///////////////////////////////////////////////////
  //<editor-fold desc="ngOnInit supporting methods">
  ///////////////////////////////////////////////////
  private loadWatchDog(): void {
    if (this.dog === undefined) {
      return;
    }
    this.controlsForm.id.setValue(this.dog.id);
    this.controlsForm.company.setValue(this.dog.company);
    this.controlsForm.originSku.setValue(this.dog.originSku);
    this.controlsForm.productName.setValue(this.dog.productName);
    this.controlsForm.priceAlarm.setValue(this.dog.priceAlarm);
  }

  private disableControlls(): void {
    this.controlsForm.id.disable();
    this.controlsForm.company.disable();
    this.controlsForm.originSku.disable();
    this.controlsForm.productName.disable();
  }

  private bindToSelectedProduct(): void {
    this.messageService.getSelectedProduct().pipe(takeWhile(() => this.alive))
      .subscribe(product => {
        console.log("catching product:"); // TODO delete it
        console.log(product); // TODO delete it

        this.selectedProduct = product;
        this.loadSelectedProduct(product);
      });
  }

  private loadSelectedProduct(selectedProduct: ProductDto) {
    if (selectedProduct === undefined || this.dog) {
      return;
    }
    this.controlsForm.company.setValue(selectedProduct.company);
    this.controlsForm.originSku.setValue(selectedProduct.originSku);
    this.controlsForm.productName.setValue(selectedProduct.name);
  }

  ///////////////////////////////////////////////////
  //</editor-fold>
  ///////////////////////////////////////////////////

  private save(): void {
    let dog = this.fillDog(null);

    this.searchService.findProductByNameAndSku(dog.company, dog.originSku, dog.productName)
      .subscribe(value => {
        console.log(value); // TODO delete it

        this.watchDogService.addWatchDog(dog).subscribe({
          next: value => {
            console.log(value); // TODO delete it
            this.funcsService.redirectTo();
          },
          error: err => {
            const tmp = new HttpErrorResponse(err);
            console.log(tmp.message); // TODO delete it?
            console.error(tmp);
          }
        });
      });

  }

  private update(): void {
    let dog = this.fillDog(this.controlsForm.id.value);

    this.watchDogService.updateWatchDog(dog).subscribe(updatedDog => {
      console.log(updatedDog); // TODO delete it
      this.dog = updatedDog;
      this.detailForm.reset(this.detailForm.value);
      this.loadWatchDog();
    });
  }

  private fillDog(idArg: number | null) {
    let dog: WatchDogDto = {
      id: idArg,
      company: this.controlsForm.company.value!,
      originSku: this.controlsForm.originSku.value!,
      productName: this.controlsForm.productName.value!,
      productUrl: this.dog?.productUrl,
      priceAlarm: this.controlsForm.priceAlarm.value!,
    };
    return dog;
  }
}

