import {AfterViewInit, ChangeDetectorRef, Component, ElementRef, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {FormBuilder} from "@angular/forms";
import {map, Observable, takeWhile, tap} from "rxjs";
import {ProductDto} from "../../../dto/ProductDto";
import {MatOptionSelectionChange} from "@angular/material/core";
import {MessageService} from "../../../services/message.service";
import {AggregatedSearchService} from "../../../services/rest/aggregated-search.service";
import {GroceryCredentialDto} from "../../../dto/GroceryCredentialDto";
import {GroceryCredentialService} from "../../../services/rest/grocery-credential-service.service";
import {MatAutocomplete, MatAutocompleteTrigger} from "@angular/material/autocomplete";

@Component({
  selector: 'app-product-search',
  templateUrl: './product-search.component.html',
  styleUrl: './product-search.component.scss',
})
export class ProductSearchComponent implements OnInit, OnDestroy, AfterViewInit {
  alive: boolean = true;

  fetchedProduct$: Observable<ProductDto[]> | undefined;

  credentialDtos: GroceryCredentialDto[] | undefined;

  @ViewChild('myInputElem')
  myInputElem: ElementRef | undefined;

  @ViewChild('myFilterElem')
  myFilterElem: ElementRef | undefined;

  @ViewChild('productSearchAuto')
  myAutoElem: MatAutocomplete | undefined;

  @ViewChild(MatAutocompleteTrigger)
  autocomplete: MatAutocompleteTrigger | undefined;

  searchForm = this.fb.group({
    searchInput: ['' as string],
    searchFilter: ['' as string],
  });
  controlsForm = this.searchForm.controls;

  constructor(private searchService: AggregatedSearchService,
              private fb: FormBuilder,
              private messageService: MessageService,
              private credentialService: GroceryCredentialService,
              private changeDetectorRef: ChangeDetectorRef) {
  }

  ngOnInit(): void {
    this.loadAllCredentials();
  }

  ngAfterViewInit(): void {
    this.listenToMsgRequests();
  }

  ngOnDestroy() {
    this.alive = false;
  }

  private loadAllCredentials(): void {
    this.credentialService.getAllCredentials().subscribe(credentials => {
      this.credentialDtos = credentials;
    });
  }

  private listenToMsgRequests() {
    this.messageService.getProductAltRequest().pipe(takeWhile(() => this.alive))
      .subscribe(product => {
        console.log("catching request:");
        console.log(product);

        this.controlsForm.searchInput.setValue(product.name);
        this.controlsForm.searchFilter.setValue(product.company);

        this.fetchedProduct$ = this.searchService.findProductsByName(product.name, product.company).pipe(
          tap(products => {
            console.log("FETCHED NOW");
            console.log(products);

            if (products.length === 0) {
              alert("No products found.");
            }

            if (this.myInputElem) {
              this.myInputElem.nativeElement.value = product.name;
            }

            if (this.myFilterElem) {
              this.myFilterElem.nativeElement.innerHTML = "FILTER&nbsp;OUT: " + product.company;
            }

            if (this.myAutoElem) {
              this.myAutoElem.showPanel = true;
            }

            if (this.autocomplete) {
              console.log("this.autocomplete.autocomplete.options.toArray():");
              console.log(this.autocomplete.autocomplete.options.toArray());
              this.autocomplete.openPanel();
            }

            // this.changeDetectorRef.detectChanges();
          })
        );

        // this.changeDetectorRef.detectChanges();
      })
  }

  clearSearch($event: MouseEvent) {
    $event.stopPropagation();

    this.controlsForm.searchInput.setValue('');
    this.controlsForm.searchFilter.setValue('');

    if (this.myFilterElem) {
      this.myFilterElem.nativeElement.innerHTML = '';
    }
    this.autocomplete?.closePanel();
  }

  displayFn(prod: ProductDto): string {
    return prod && prod.name ? prod.name : '';
  }

  findProduct($event: Event): void {
    $event.stopPropagation();

    if ($event instanceof PointerEvent && $event.pointerType !== 'mouse') {
      return; // PointerEvent occurs when user pressed enter (javascript bug?)
    }

    const val: any = this.controlsForm.searchInput.value;
    if (typeof val !== "string") {
      return;
    }

    let productName = val?.trim() ?? '';
    if (productName === '') {
      return;
    }

    console.log("Try to find:" + productName); // TODO - delete it

    this.controlsForm.searchFilter.setValue('');

    if (this.myFilterElem) {
      this.myFilterElem.nativeElement.innerHTML = '';
    }

    this.fetchedProduct$ = this.searchService.findProductsByName(productName).pipe(
      tap(products => {
        if (products.length === 0) {
          alert("No products found.");
        }
      })
    );
    this.fetchedProduct$ = this.roundProductsStream(this.fetchedProduct$);
  }

  selectedOption($event: MatOptionSelectionChange<ProductDto> | ProductDto): void {
    let selectedProduct: ProductDto;

    if ($event instanceof MatOptionSelectionChange) {

      if (!$event.source.selected) {
        return;
      }
      selectedProduct = $event.source.value;
    } else {
      selectedProduct = $event;
    }

    this.messageService.setSelectedProduct(selectedProduct);
  }

  private roundProductsStream(product$: Observable<ProductDto[]>) {
    return product$.pipe(
      map(products => products
        .map(product => {
          // copy to a new object
          return {
            ...product,
            pricePerUnitFull: Math.round(product.pricePerUnitFull),
            priceFull: Math.round(product.priceFull)
          };
        })
      ));
  }
}
