import {Component, Input, OnInit} from '@angular/core';
import {FormBuilder, Validators} from "@angular/forms";
import {FuncsService} from "../../../services/funcs.service";
import {HttpErrorResponse} from "@angular/common/http";
import {ShoppingListDto} from "../../../dto/ShoppingListDto";
import {ShoppingListService} from "../../../services/rest/shopping-list.service";
import {ShoppingListEnum} from "../../../util/ShoppingListEnum";

@Component({
  selector: 'app-shopping-lists-detail',
  templateUrl: './shopping-lists-detail.component.html',
  styleUrl: './shopping-lists-detail.component.scss'
})
export class ShoppingListsDetailComponent implements OnInit {

  @Input()
  aList?: ShoppingListDto;

  @Input({required: true})
  shouldHide!: boolean;

  detailForm = this.fb.group({
    id: [null as number | null],
    name: ['' as string, {validators: [Validators.required], nonNullable: true}]
  });
  controlsForm = this.detailForm.controls;

  constructor(private shoppingListService: ShoppingListService,
              private fb: FormBuilder,
              private funcsService: FuncsService) {
  }

  ngOnInit(): void {
    this.loadShoppingList();
  }

  isSaveEnabled(): boolean {
    let formValid: boolean = this.detailForm.valid
      && this.controlsForm.name.value !== '';

    let formChanged: boolean = this.detailForm.dirty;

    return formValid && formChanged;
  }

  isDeleteEnabled(): boolean {
    return this.controlsForm.id.value !== null;
  }

  saveOrUpdate(): void {
    if (this.controlsForm.id.value === null) {
      this.save();
    } else {
      this.update();
    }
  }

  delete(): void {
    if (!this.isDeleteEnabled()) {
      return;
    }
    this.shoppingListService.deleteShoppingList(this.controlsForm.id.value!).subscribe(value => {
      this.funcsService.redirectTo();
    });
  }

  ///////////////////////////////////////////////////
  //<editor-fold desc="ngOnInit supporting methods">
  ///////////////////////////////////////////////////
  private loadShoppingList(): void {
    if (this.aList !== undefined) {
      this.controlsForm.id.setValue(this.aList.id);
      this.controlsForm.name.setValue(this.aList.name);
    }
  }

  ///////////////////////////////////////////////////
  //</editor-fold>
  ///////////////////////////////////////////////////

  private save(): void {
    let shoppingListDto = this.fillShoppingList(null);

    this.shoppingListService.addShoppingList(shoppingListDto).subscribe({
      next: value => {
        console.log(value); // TODO delete it
        this.funcsService.redirectTo();
      },
      error: err => {
        const tmp = new HttpErrorResponse(err);
        console.log(tmp.message); // TODO delete it?
        console.error(tmp);
      }
    });
  }

  private update(): void {
    let listDto = this.fillShoppingList(this.controlsForm.id.value);

    this.shoppingListService.updateShoppingList(listDto).subscribe(value => {
      console.log(value); // TODO delete it
      this.aList = value;
      this.detailForm.reset(this.detailForm.value);
      this.loadShoppingList();
    });
  }

  private fillShoppingList(idArg: number | null) {
    let listDto: ShoppingListDto = {
      id: idArg,
      name: this.controlsForm.name.value!,
      type: ShoppingListEnum.WISHLIST
    } as ShoppingListDto;
    return listDto;
  }

  load() {
    if (!confirm("Are you sure, you want to load this list into your carts?")) {
      return;
    }
    const shoppingListId = this.controlsForm.id.value;
    if (!shoppingListId) {
      return;
    }

    this.shoppingListService.copyShoppingListToCarts(shoppingListId).subscribe({
      complete: () => this.funcsService.redirectTo("/carts")
    });
  }

}
