import {Injectable} from '@angular/core';
import {catchError, map, Observable, of, tap} from "rxjs";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {GroceryCredentialDto} from "../../dto/GroceryCredentialDto";
import {Constants} from "../../util/Constants";
import {FuncsService} from "../funcs.service";

@Injectable({
  providedIn: 'root'
})
export class GroceryCredentialService {
  private url: string = Constants.SERVER_URL + '/grocery-credential';

  httpOptions = {
    headers: new HttpHeaders({'Content-Type': 'application/json'})
  };

  constructor(
    private http: HttpClient,
    private funcsService: FuncsService
  ) {
  }

  /** GET credentials from the server */
  getAllCredentials(): Observable<GroceryCredentialDto[]> {
    return this.http.get<GroceryCredentialDto[]>(this.url)
      .pipe(
        tap(_ => this.funcsService.log('fetched credentials')),
        catchError(this.funcsService.handleError<GroceryCredentialDto[]>('getCredentials', []))
      );
  }

  /** GET credential by id. Will 404 if id not found */
  getCredentialById(id: number): Observable<GroceryCredentialDto> {
    const url = `${this.url}/${id}`;
    return this.http.get<GroceryCredentialDto>(url).pipe(
      tap(_ => this.funcsService.log(`fetched credential id=${id}`)),
      catchError(this.funcsService.handleError<GroceryCredentialDto>(`getCredential id=${id}`))
    );
  }

  //////// Save methods //////////

  /** POST: add a new credential to the server */
  addCredential(credential: GroceryCredentialDto): Observable<GroceryCredentialDto> {
    return this.http.post<GroceryCredentialDto>(this.url, credential, this.httpOptions).pipe(
      tap((newCredential: GroceryCredentialDto) => this.funcsService.log(`added credential id=${newCredential.id}`)),
      catchError(this.funcsService.handleError<GroceryCredentialDto>('addCredential'))
    );
  }

  /** PUT: update the credential on the server */
  updateCredential(credential: GroceryCredentialDto): Observable<GroceryCredentialDto> {
    const url = `${this.url}/${credential.id}`;

    return this.http.put<GroceryCredentialDto>(url, credential, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`updated credential id=${credential.id}`)),
      catchError(this.funcsService.handleError<any>('updateCredential'))
    );
  }

  /** DELETE: delete the credential from the server */
  deleteCredential(id: number): Observable<void> {
    const url = `${this.url}/${id}`;

    return this.http.delete<void>(url, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`deleted credential id=${id}`)),
      catchError(this.funcsService.handleError<void>('deleteCredential'))
    );
  }

  //////// Others methods //////////

  /** GET credential by id. Return `undefined` when id not found */
  getCredentialNo404<Data>(id: number): Observable<GroceryCredentialDto> {
    const url = `${this.url}/?id=${id}`;
    return this.http.get<GroceryCredentialDto[]>(url)
      .pipe(
        map(credentials => credentials[0]), // returns a {0|1} element array
        tap(h => {
          const outcome = h ? 'fetched' : 'did not find';
          this.funcsService.log(`${outcome} credential id=${id}`);
        }),
        catchError(this.funcsService.handleError<GroceryCredentialDto>(`getCredential id=${id}`))
      );
  }

  /* GET credentials whose name contains search term */
  searchCredentials(term: string): Observable<GroceryCredentialDto[]> {
    if (!term.trim()) {
      // if not search term, return empty credential array.
      return of([]);
    }
    return this.http.get<GroceryCredentialDto[]>(`${this.url}/?name=${term}`).pipe(
      tap(x => x.length ?
        this.funcsService.log(`found credentials matching "${term}"`) :
        this.funcsService.log(`no credentials matching "${term}"`)),
      catchError(this.funcsService.handleError<GroceryCredentialDto[]>('searchCredentials', []))
    );
  }
}
