import {Constants} from "../../util/Constants";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {FuncsService} from "../funcs.service";
import {ItemDto} from "../../dto/ItemDto";
import {catchError, map, mergeMap, Observable, of, take, tap} from "rxjs";

/* error cause urlArg const
@Injectable({
  providedIn: 'root'
})
*/
export class GroceryRestService {
  private readonly url: string;
  private readonly urlBackend: string;

  httpOptions = {
    headers: new HttpHeaders({'Content-Type': 'application/json'})
  };

  constructor(private http: HttpClient,
              private funcsService: FuncsService,
              urlArg: string) {
    this.url = Constants.SERVER_URL + '/api/' + urlArg;
    this.urlBackend = '/api/' + urlArg;
  }

  /** Login cartItems from the server */
  login(): Observable<boolean> {
    return this.http.get<boolean>(this.url + '/login')
      .pipe(
        tap(_ => this.funcsService.log('try to login')),
        catchError(this.funcsService.handleError<boolean>('try to login'))
      );
  }

  /** GET cartItems from the server */
  getAllCartItems(): Observable<ItemDto[]> {
    return this.http.get<ItemDto[]>(`${this.url}/cart`)
      .pipe(
        tap(_ => this.funcsService.log(`fetched cartItems (${this.urlBackend}/cart)`)),
        catchError(this.funcsService.handleError<ItemDto[]>('getCartItems', []))
      )
  }

  /** GET cartItem by id. Will 404 if id not found */
  getCartItemById(id: number): Observable<ItemDto> {
    const url = `${this.url}/${id}`;
    return this.http.get<ItemDto>(url).pipe(
      tap(_ => this.funcsService.log(`fetched cartItem id=${id}`)),
      catchError(this.funcsService.handleError<ItemDto>(`getCartItem id=${id}`))
    );
  }

  //////// Save methods //////////

  /** POST: add a new cartItem to the server */
  addCartItem(cartItem: ItemDto): Observable<ItemDto> {
    const url = `${this.url}/cart`;

    return this.http.post<ItemDto[]>(url, cartItem, this.httpOptions).pipe(
      mergeMap(items => {
        return items.filter(item =>
          item.originSku === cartItem.originSku);
      }),
      take(1),
      tap((newCartItem: ItemDto) => this.funcsService.log(`added cartItem originSku=${newCartItem.originSku}`)),
      catchError(this.funcsService.handleError<ItemDto>('addCartItem'))
    );
  }

  /** PUT: update the cartItem on the server */
  updateCartItem(cartItem: ItemDto): Observable<ItemDto> {
    const url = `${this.url}/cart`;//${cartItem.itemId}`;

    return this.http.put<ItemDto[]>(url, cartItem, this.httpOptions).pipe(
      mergeMap(items => {
        return items.filter(item =>
          item.originItemId === cartItem.originItemId);
      }),
      take(1),
      tap(_ => this.funcsService.log(`updated cartItem originItemId=${cartItem.originItemId}`)),
      catchError(this.funcsService.handleError<ItemDto>('updateCartItem'))
    );
  }

  /** DELETE: delete the cartItem from the server */
  deleteAllCartItems(): Observable<void> {
    const url = `${this.url}/cart/all`;

    return this.http.delete<void>(url, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`deleted all cartItems (${this.urlBackend})`)),
      catchError(this.funcsService.handleError<void>('deleteAllCartItems'))
    );
  }

  //////// Others methods //////////

  /** GET cartItem by id. Return `undefined` when id not found */
  getCartItemNo404<Data>(id: number): Observable<ItemDto> {
    const url = `${this.url}/?id=${id}`;
    return this.http.get<ItemDto[]>(url)
      .pipe(
        map(cartItems => cartItems[0]), // returns a {0|1} element array
        tap(h => {
          const outcome = h ? 'fetched' : 'did not find';
          this.funcsService.log(`${outcome} cartItem id=${id}`);
        }),
        catchError(this.funcsService.handleError<ItemDto>(`getCartItem id=${id}`))
      );
  }

  /* GET cartItems whose name contains search term */
  searchCartItems(term: string): Observable<ItemDto[]> {
    if (!term.trim()) {
      // if not search term, return empty cartItem array.
      return of([]);
    }
    return this.http.get<ItemDto[]>(`${this.url}/?name=${term}`).pipe(
      tap(x => x.length ?
        this.funcsService.log(`found cartItems matching "${term}"`) :
        this.funcsService.log(`no cartItems matching "${term}"`)),
      catchError(this.funcsService.handleError<ItemDto[]>('searchCartItems', []))
    );
  }
  
}
