import {Injectable} from '@angular/core';
import {catchError, map, Observable, of, tap} from "rxjs";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {Constants} from "../../util/Constants";
import {FuncsService} from "../funcs.service";
import {ItemDto} from "../../dto/ItemDto";

@Injectable({
  providedIn: 'root'
})
export class ShoppingListItemService {
  private url: string = Constants.SERVER_URL + '/shopping-lists';

  httpOptions = {
    headers: new HttpHeaders({'Content-Type': 'application/json'})
  };

  constructor(
    private http: HttpClient,
    private funcsService: FuncsService
  ) {
  }

  /** GET shoppingListItems from the server */
  getAllShoppingListItems(listId: number): Observable<ItemDto[]> {
    const url = `${this.url}/${listId}/items`;
    return this.http.get<ItemDto[]>(url)
      .pipe(
        tap(_ => this.funcsService.log('fetched shoppingListItems')),
        catchError(this.funcsService.handleError<ItemDto[]>('getShoppingListItems', []))
      );
  }

  /** GET shoppingListItem by id. Will 404 if id not found */
  getShoppingListItemById(listId: number, itemId: number): Observable<ItemDto> {
    const url = `${this.url}/${listId}/items/${itemId}`;
    return this.http.get<ItemDto>(url).pipe(
      tap(_ => this.funcsService.log(`fetched shoppingListItem listId=${listId}, itemId=${itemId}`)),
      catchError(this.funcsService.handleError<ItemDto>(`getShoppingListItem listId=${listId}, itemId=${itemId}`))
    );
  }

  //////// Save methods //////////

  /** POST: add a new shoppingListItem to the server */
  addShoppingListItem(listId: number, shoppingListItem: ItemDto): Observable<ItemDto> {
    const url = `${this.url}/${listId}/items`;

    return this.http.post<ItemDto>(url, shoppingListItem, this.httpOptions).pipe(
      tap((newShoppingListItem: ItemDto) => this.funcsService.log(`added shoppingListItem id=${newShoppingListItem.itemId}`)),
      catchError(this.funcsService.handleError<ItemDto>('addShoppingListItem'))
    );
  }

  /** PUT: update the shoppingListItem on the server */
  updateShoppingListItem(listId: number, shoppingListItem: ItemDto): Observable<ItemDto> {
    const url = `${this.url}/${listId}/items/${shoppingListItem.itemId}`;

    return this.http.put<ItemDto>(url, shoppingListItem, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`updated shoppingListItem id=${shoppingListItem.itemId}`)),
      catchError(this.funcsService.handleError<ItemDto>('updateShoppingListItem'))
    );
  }

  /** DELETE: delete the shoppingListItem from the server */
  deleteShoppingListItem(listId: number, itemId: number): Observable<void> {
    const url = `${this.url}/${listId}/items/${itemId}`;

    return this.http.delete<void>(url, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`deleted shoppingListItem listId=${listId}`)),
      catchError(this.funcsService.handleError<void>('deleteShoppingListItem'))
    );
  }


}
