import {Injectable} from '@angular/core';
import {catchError, map, Observable, of, tap} from "rxjs";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {Constants} from "../../util/Constants";
import {FuncsService} from "../funcs.service";
import {ShoppingListDto} from "../../dto/ShoppingListDto";

@Injectable({
  providedIn: 'root'
})
export class ShoppingListService {
  private url: string = Constants.SERVER_URL + '/shopping-lists';

  httpOptions = {
    headers: new HttpHeaders({'Content-Type': 'application/json'})
  };

  constructor(
    private http: HttpClient,
    private funcsService: FuncsService
  ) {
  }

  /** GET shoppingLists from the server */
  getAllShoppingLists(): Observable<ShoppingListDto[]> {
    return this.http.get<ShoppingListDto[]>(this.url)
      .pipe(
        tap(_ => this.funcsService.log('fetched shoppingLists')),
        catchError(this.funcsService.handleError<ShoppingListDto[]>('getShoppingLists', []))
      );
  }

  /** GET shoppingList by id. Will 404 if id not found */
  getShoppingListById(id: number): Observable<ShoppingListDto> {
    const url = `${this.url}/${id}`;
    return this.http.get<ShoppingListDto>(url).pipe(
      tap(_ => this.funcsService.log(`fetched shoppingList id=${id}`)),
      catchError(this.funcsService.handleError<ShoppingListDto>(`getShoppingList id=${id}`))
    );
  }

  //////// Save methods //////////

  /** POST: add a new shoppingList to the server */
  addShoppingList(shoppingList: ShoppingListDto): Observable<ShoppingListDto> {
    return this.http.post<ShoppingListDto>(this.url, shoppingList, this.httpOptions).pipe(
      tap((newShoppingList: ShoppingListDto) => this.funcsService.log(`added shoppingList id=${newShoppingList.id}`)),
      catchError(this.funcsService.handleError<ShoppingListDto>('addShoppingList'))
    );
  }

  /** PUT: update the shoppingList on the server */
  updateShoppingList(shoppingList: ShoppingListDto): Observable<ShoppingListDto> {
    const url = `${this.url}/${shoppingList.id}`;

    return this.http.put<ShoppingListDto>(url, shoppingList, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`updated shoppingList id=${shoppingList.id}`)),
      catchError(this.funcsService.handleError<ShoppingListDto>('updateShoppingList'))
    );
  }

  /** DELETE: delete the shoppingList from the server */
  deleteShoppingList(id: number): Observable<void> {
    const url = `${this.url}/${id}`;

    return this.http.delete<void>(url, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`deleted shoppingList id=${id}`)),
      catchError(this.funcsService.handleError<void>('deleteShoppingList'))
    );
  }

  //////// Special methods //////////

  /** POST: copy carts to your selected shoppingList */
  copyCartsToShoppingList(id: number): Observable<ShoppingListDto> {
    const url = `${this.url}/${id}/copy-from-carts`;

    return this.http.post<ShoppingListDto>(url, null, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`copyCartsToShoppingList with id=${id}`)),
      catchError(this.funcsService.handleError<ShoppingListDto>('copyCartsToShoppingList'))
    );
  }

  /** POST: copy your selected shoppingList to carts */
  copyShoppingListToCarts(id: number): Observable<ShoppingListDto> {
    const url = `${this.url}/${id}/copy-to-carts`;

    return this.http.post<ShoppingListDto>(url, null, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`copyShoppingListToCarts with id=${id}`)),
      catchError(this.funcsService.handleError<ShoppingListDto>('copyShoppingListToCarts'))
    );
  }

  //////// Others methods //////////

  /** GET shoppingList by id. Return `undefined` when id not found */
  getShoppingListNo404<Data>(id: number): Observable<ShoppingListDto> {
    const url = `${this.url}/?id=${id}`;
    return this.http.get<ShoppingListDto[]>(url)
      .pipe(
        map(shoppingLists => shoppingLists[0]), // returns a {0|1} element array
        tap(h => {
          const outcome = h ? 'fetched' : 'did not find';
          this.funcsService.log(`${outcome} shoppingList id=${id}`);
        }),
        catchError(this.funcsService.handleError<ShoppingListDto>(`getShoppingList id=${id}`))
      );
  }

  /* GET shoppingLists whose name contains search term */
  searchShoppingLists(term: string): Observable<ShoppingListDto[]> {
    if (!term.trim()) {
      // if not search term, return empty shoppingList array.
      return of([]);
    }
    return this.http.get<ShoppingListDto[]>(`${this.url}/?name=${term}`).pipe(
      tap(x => x.length ?
        this.funcsService.log(`found shoppingLists matching "${term}"`) :
        this.funcsService.log(`no shoppingLists matching "${term}"`)),
      catchError(this.funcsService.handleError<ShoppingListDto[]>('searchShoppingLists', []))
    );
  }

}
