import {Injectable} from '@angular/core';
import {catchError, map, Observable, of, tap} from "rxjs";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {WatchDogDto} from "../../dto/WatchDogDto";
import {Constants} from "../../util/Constants";
import {FuncsService} from "../funcs.service";

@Injectable({
  providedIn: 'root'
})
export class WatchDogService {
  private url: string = Constants.SERVER_URL + '/watchdog';

  httpOptions = {
    headers: new HttpHeaders({'Content-Type': 'application/json'})
  };

  constructor(
    private http: HttpClient,
    private funcsService: FuncsService
  ) {
  }

  /** GET watchDogs from the server */
  getAllWatchDogs(): Observable<WatchDogDto[]> {
    return this.http.get<WatchDogDto[]>(this.url)
      .pipe(
        tap(_ => this.funcsService.log('fetched watchDogs')),
        catchError(this.funcsService.handleError<WatchDogDto[]>('getWatchDogs', []))
      );
  }

  /** GET watchDog by id. Will 404 if id not found */
  getWatchDogById(id: number): Observable<WatchDogDto> {
    const url = `${this.url}/${id}`;
    return this.http.get<WatchDogDto>(url).pipe(
      tap(_ => this.funcsService.log(`fetched watchDog id=${id}`)),
      catchError(this.funcsService.handleError<WatchDogDto>(`getWatchDog id=${id}`))
    );
  }

  //////// Save methods //////////

  /** POST: add a new watchDog to the server */
  addWatchDog(watchDog: WatchDogDto): Observable<WatchDogDto> {
    return this.http.post<WatchDogDto>(this.url, watchDog, this.httpOptions).pipe(
      tap((newWatchDog: WatchDogDto) => this.funcsService.log(`added watchDog id=${newWatchDog.id}`)),
      catchError(this.funcsService.handleError<WatchDogDto>('addWatchDog'))
    );
  }

  /** PUT: update the watchDog on the server */
  updateWatchDog(watchDog: WatchDogDto): Observable<WatchDogDto> {
    const url = `${this.url}/${watchDog.id}`;

    return this.http.put<WatchDogDto>(url, watchDog, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`updated watchDog id=${watchDog.id}`)),
      catchError(this.funcsService.handleError<any>('updateWatchDog'))
    );
  }

  /** DELETE: delete the watchDog from the server */
  deleteWatchDog(id: number): Observable<void> {
    const url = `${this.url}/${id}`;

    return this.http.delete<void>(url, this.httpOptions).pipe(
      tap(_ => this.funcsService.log(`deleted watchDog id=${id}`)),
      catchError(this.funcsService.handleError<void>('deleteWatchDog'))
    );
  }

  //////// Others methods //////////

  /** GET watchDog by id. Return `undefined` when id not found */
  getWatchDogNo404<Data>(id: number): Observable<WatchDogDto> {
    const url = `${this.url}/?id=${id}`;
    return this.http.get<WatchDogDto[]>(url)
      .pipe(
        map(watchDogs => watchDogs[0]), // returns a {0|1} element array
        tap(h => {
          const outcome = h ? 'fetched' : 'did not find';
          this.funcsService.log(`${outcome} watchDog id=${id}`);
        }),
        catchError(this.funcsService.handleError<WatchDogDto>(`getWatchDog id=${id}`))
      );
  }

  /* GET watchDogs whose name contains search term */
  searchWatchDogs(term: string): Observable<WatchDogDto[]> {
    if (!term.trim()) {
      // if not search term, return empty watchDog array.
      return of([]);
    }
    return this.http.get<WatchDogDto[]>(`${this.url}/?name=${term}`).pipe(
      tap(x => x.length ?
        this.funcsService.log(`found watchDogs matching "${term}"`) :
        this.funcsService.log(`no watchDogs matching "${term}"`)),
      catchError(this.funcsService.handleError<WatchDogDto[]>('searchWatchDogs', []))
    );
  }

}
